package com.ruoyi.web.controller.app;

import cn.hutool.core.util.StrUtil;
import com.ruoyi.common.core.controller.BaseController;
import com.ruoyi.common.core.domain.AjaxResult;
import com.ruoyi.common.core.domain.vo.FeeRuleVo;
import com.ruoyi.common.core.page.TableDataInfo;
import com.ruoyi.common.utils.CommonUtil;
import com.ruoyi.system.domain.*;
import com.ruoyi.system.service.*;
import com.ruoyi.web.controller.common.WebSocket;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.web.bind.annotation.*;

import java.util.List;

/**
 * app接口（不需要登录的）
 *  校验
 * @author ruoyi
 */
@RestController
@RequestMapping("/app")
public class AppController extends BaseController
{
    @Autowired
    private IAsArticleService asArticleService;

    @Autowired
    private IEtOperatingAreaService etOperatingAreaService;

    @Autowired
    private IEtParkingAreaService etParkingAreaService;

    @Autowired
    private IAsDeviceService asDeviceService;

    @Autowired
    private WebSocket webSocket;

    @Autowired
    private IEtFeeRuleService etFeeRuleService;

    @Autowired
    private ISysConfigService sysConfigService;

    @Autowired
    private IEtOrderService etOrderService;



    /**
     * 查询文章列表
     */
    @GetMapping("/article/list")
    public TableDataInfo articleList(EtArticle etArticle)
    {
        startPage();
        List<EtArticle> list = asArticleService.selectAsArticleList(etArticle);
        return getDataTable(list);
    }

    /**
     * 获取文章详细信息
     */
    @GetMapping(value = "/article/{articleId}")
    public AjaxResult getArticleInfo(@PathVariable("articleId") Long articleId)
    {
        return success(asArticleService.selectAsArticleByArticleId(articleId));
    }


    /**
     * 查询运营区列表
     */
    @GetMapping("/area/list")
    public TableDataInfo list(EtOperatingArea etOperatingArea)
    {
        etOperatingArea.setStatus("0");//运营区状态：0-运营中；1-停运
        startPage();
        List<EtOperatingArea> list = etOperatingAreaService.selectEtOperatingAreaList(etOperatingArea);
        return getDataTable(list);
    }

    /**
     * 获取运营区详细信息
     */
    @GetMapping(value = "/area/{areaId}")
    public AjaxResult getInfo(@PathVariable("areaId") Long areaId)
    {
        EtOperatingArea area = etOperatingAreaService.selectEtOperatingAreaByAreaId(areaId);
        return success(area);
    }


    /**
     * 根据定位获取运营区详细信息
     */
    @GetMapping(value = "/area/info")
    public AjaxResult areaInfo(String longitude,String latitude)
    {
        if(StrUtil.isBlank(longitude) || StrUtil.isBlank(latitude)){
            logger.info("没有经纬度参数：【longitude={}】，【latitude={}】",longitude,latitude);
            return error("请传经纬度参数"+"【longitude="+longitude+"】，【latitude="+latitude+"】");
        }
        EtOperatingArea area = etOperatingAreaService.getAreaInfoByLocation(longitude,latitude);
        return success(area);
    }

    /**
     * 查询停车区列表
     */
    @GetMapping("/parking/list")
    public TableDataInfo list(EtParkingArea etParkingArea)
    {
        startPage();
        List<EtParkingArea> list = etParkingAreaService.selectEtParkingAreaList(etParkingArea);
        return getDataTable(list);
    }

    /**
     * 获取停车区详细信息
     */
    @GetMapping(value = "/parking/{parkingId}")
    public AjaxResult getParkingInfo(@PathVariable("parkingId") Long parkingId)
    {
        return success(etParkingAreaService.selectEtParkingAreaByParkingId(parkingId));
    }


    /**
     * 根据定位获取运营区信息，并返回所有车辆定位
     */
    @GetMapping(value = "/vehicleLocalization")
    public AjaxResult vehicleLocalization(String longitude,String latitude)
    {
        if(StrUtil.isBlank(longitude) || StrUtil.isBlank(latitude)){
            logger.info("没有经纬度参数：【longitude={}】，【latitude={}】",longitude,latitude);
            return error("请传经纬度参数"+"【longitude="+longitude+"】，【latitude="+latitude+"】");
        }
//        webSocket.SendMessage("需要发送的消息", "识别唯一session");
        List<AsDevice> asDevices = asDeviceService.vehicleLocalization(longitude,latitude);
        return success(asDevices);
    }


    /**
     * 根据运营区获取收费方式列表
     */
    @GetMapping("/fee/list")
    public AjaxResult feeList(EtFeeRule etFeeRule)
    {
        List<EtFeeRule> list = etFeeRuleService.selectEtFeeRuleListByAreaId(etFeeRule.getAreaId());
        return success(list);
    }

    /**
     * 获取计费套餐详细信息
     */
    @GetMapping(value = "/fee/{ruleId}")
    public AjaxResult getFeeInfo(@PathVariable("ruleId") Long ruleId)
    {
        return success(etFeeRuleService.selectEtFeeRuleByRuleId(ruleId));
    }

//    /**
//     * 获取计费规则  多少分钟内  里程数
//     */
//    @GetMapping(value = "/fee/rule")
//    public AjaxResult getRule()
//    {
//        String appointmentServiceFee = sysConfigService.selectConfigByKey("appointment.service.fee");//预约服务费
//        String dispatchFee = sysConfigService.selectConfigByKey("dispatch.fee");//调度费
//        String vehicleManagementFee = sysConfigService.selectConfigByKey("vehicle.management.fee");//车辆管理费
//        String startingPrice = sysConfigService.selectConfigByKey("starting.price");//起步价
//        String startingHowManyMinutes = sysConfigService.selectConfigByKey("starting.how.many.minutes");//多少分钟内按起步价
//        String timeFee = sysConfigService.selectConfigByKey("time.fee");//时长费
//        String timeMinutes = sysConfigService.selectConfigByKey("time.minutes");//时长分钟
//        String timeoutMinutes = sysConfigService.selectConfigByKey("timeout.minutes");//预约超时保留分钟
//        String deposit = sysConfigService.selectConfigByKey("deposit");//押金
//        String isMeal = sysConfigService.selectConfigByKey("is.meal");//是否有套餐 是否有套餐：0-没有套餐；1-有套餐
//        String cappingFee = sysConfigService.selectConfigByKey("order.capping.fee");//24小时内封顶费用
//        String forceParkingArea = sysConfigService.selectConfigByKey("force.parking.area");//是否强制在停车区内还车
//        FeeRuleVo build = FeeRuleVo.builder()
//                .appointmentServiceFee(appointmentServiceFee)
//                .dispatchFee(dispatchFee)
//                .vehicleManagementFee(vehicleManagementFee)
//                .startingPrice(startingPrice)
//                .startingHowManyMinutes(startingHowManyMinutes)
//                .timeFee(timeFee)
//                .timeMinutes(timeMinutes)
//                .timeoutMinutes(timeoutMinutes)
//                .deposit(deposit)
//                .isMeal(isMeal)
//                .cappingFee(cappingFee)
//                .forceParkingArea(forceParkingArea)
//                .build();
//        return success(build);
//    }


    /**
     * 响铃寻车
     */
    @PostMapping("/device/ring")
    public AjaxResult ring(String sn)
    {
       Boolean i =asDeviceService.ring(sn);
        return success(i);
    }

    /**
     * 根据sn号查询车辆实时信息
     */
    @GetMapping("/device/info")
    public AjaxResult deviceInfo(String sn)
    {
        AsDevice device = asDeviceService.selectAsDeviceBySn(sn);
        return success(device);
    }

    /**
     * 查询当前用户是否有正在进行中的订单
     */
    @PostMapping("/user/isInOrder")
    public AjaxResult isInOrder(Long userId)
    {
        List<EtOrder> orders = etOrderService.isInOrder(userId,null);
        return success(orders);
    }

    /**
     * 根据条件模糊查询车辆信息
     * type:1-sn,2-mac,3-车牌号
     */
    @GetMapping("/device/search")
    public AjaxResult deviceSearch(String key,String type)
    {
        logger.info("根据条件模糊查询车辆信息：【key="+key+"】，【type="+type+"】");
        List<AsDevice> asDevices = asDeviceService.deviceSearch(key,type);
        return success(asDevices);
    }

    /**
     * 根据定位获取地址    逆地理编码
     */
    @GetMapping("/getAddress")
    public AjaxResult deviceList(String location)
    {
        logger.info("根据定位获取地址：【location="+location+"】");
        String address = CommonUtil.getAddressByGeo(location);
        logger.info("根据定位获取地址：【address="+address+"】");
        return AjaxResult.success("操作成功", address);
    }

    /**
     * 获取省市县
     */
    @GetMapping("/getDistrictList")
    public AjaxResult getDistrictList()
    {
        logger.info("【获取省市县】");
        String address = CommonUtil.getDistrictList();
//        logger.info("【获取省市县】【address="+address+"】");
        return AjaxResult.success("操作成功", address);
    }

}
