package com.ruoyi.web.controller.app;

import cn.hutool.core.util.ObjectUtil;
import cn.hutool.core.util.StrUtil;
import com.alibaba.fastjson2.JSON;
import com.ruoyi.common.annotation.Log;
import com.ruoyi.common.constant.ServiceConstants;
import com.ruoyi.common.core.controller.BaseController;
import com.ruoyi.common.core.domain.AjaxResult;
import com.ruoyi.common.core.domain.entity.AsUser;
import com.ruoyi.common.core.domain.entity.SysDictData;
import com.ruoyi.common.core.domain.model.LoginUser;
import com.ruoyi.common.core.page.TableDataInfo;
import com.ruoyi.common.enums.BusinessType;
import com.ruoyi.common.exception.ServiceException;
import com.ruoyi.common.utils.CommonUtil;
import com.ruoyi.common.utils.SecurityUtils;
import com.ruoyi.common.utils.verify.vo.IDResponse;
import com.ruoyi.framework.web.service.TokenService;
import com.ruoyi.system.domain.*;
import com.ruoyi.system.domain.response.FaultResponse;
import com.ruoyi.system.domain.response.OrderResponse;
import com.ruoyi.system.domain.vo.*;
import com.ruoyi.system.service.*;
import com.wechat.pay.java.service.payments.jsapi.model.PrepayWithRequestPaymentResponse;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.transaction.annotation.Transactional;
import org.springframework.web.bind.annotation.*;

import java.math.BigDecimal;
import java.util.ArrayList;
import java.util.List;

/**
 * app接口（需要登录校验的）
 *  校验
 * @author ruoyi
 */
@RestController
@RequestMapping("/appVerify")
public class AppVerifyController extends BaseController
{
    @Autowired
    private IEtFaultService etFaultService;

    @Autowired
    private IAsDeviceService asDeviceService;

    @Autowired
    private IEtOrderService etOrderService;

    @Autowired
    private IAsUserService asUserService;

    @Autowired
    private ISysDictDataService dictDataService;

    @Autowired
    private IEtAdminOrderService etAdminOrderService;

    @Autowired
    private IEtAdminOrderHistoryService etAdminOrderHistoryService;

    @Autowired
    private IEtParkingAreaService etParkingAreaService;

    @Autowired
    private TokenService tokenService;

    @Autowired
    private IEtOperatingAreaService etOperatingAreaService;



    /**
     * 故障上报
     */
    @Log(title = "故障上报", businessType = BusinessType.INSERT)
    @PostMapping("/fault")
    public AjaxResult add(@RequestBody EtFault etFault)
    {
        return toAjax(etFaultService.insertEtFault(etFault));
    }

    /**
     * 获取故障上报详细信息
     */
    @GetMapping(value = "/fault/{faultId}")
    public AjaxResult getFaultInfo(@PathVariable("faultId") Long faultId)
    {
        return success(etFaultService.selectEtFaultByFaultId(faultId));
    }

    /**
     * 查询故障上报列表
     */
    @GetMapping("/fault/list")
    public TableDataInfo faultList(EtFault etFault)
    {
        startPage();
        List<EtFault> list = etFaultService.selectEtFaultList(etFault);
        return getDataTable(list);
    }

    /**
     * 所有的故障类型（分车头、车身）
     */
    @GetMapping("/fault/allList")
    public AjaxResult list(SysDictData dictData)
    {
        FaultResponse faultResponse = new FaultResponse();
        List<SysDictData> list = dictDataService.selectDictDataList(dictData);
        List<SysDictData> headstock = new ArrayList<>();
        List<SysDictData> body = new ArrayList<>();
        List<SysDictData> special = new ArrayList<>();
        //根据list中的dictValue值分为三个list,dictValue值0-10，11-20，21-30
        list.forEach(item->{
            if(item.getDictValue().startsWith("0")){
                item.setListClass("车头故障");
                headstock.add(item);
            }else if(item.getDictValue().startsWith("1")){
                item.setListClass("车身故障");
                body.add(item);
            }else if(item.getDictValue().startsWith("2")){
                item.setListClass("特殊故障");
                special.add(item);
            }
        });
        faultResponse.setHeadstock(headstock);
        faultResponse.setBody(body);
        faultResponse.setSpecial(special);
        return success(faultResponse);
    }


    /**
     * 扫码/编号开锁骑行
     */
    @Log(title = "扫码/编号开锁骑行", businessType = BusinessType.UNLOCKING)
    @PostMapping("/device/snSwitch")
    public AjaxResult snSwitch(@RequestBody EtOrderVo order)
    {
        //运营时间判断
        if(!asDeviceService.isOperatingTime(order.getSn())){
            return error("不在营业时间内，不得骑行");
        }
        //根据余额和充值记录判断是否有充值过押金，没有充值过押金，提示充值押金
        if(!asUserService.checkIsDeposit(order.getUserId())){
            return error("您还未充值押金，请先充值押金");
        }
        logger.info("【扫码/编号开锁骑行】请求：{}", JSON.toJSON(order));
        if(etOrderService.isInOrder(order.getUserId(),order.getOrderNo()).size()>0){
            return error("您有未完成的订单，请先完成订单");
        }
        //低电量不得骑行判断
        if(asDeviceService.isLowBattery(order.getSn())){
            return error("低电量不得骑行");
        }
        OrderResponse orderResponse =asDeviceService.snSwitch(order);
        return success(orderResponse);
    }


    /**
     * 我的订单列表
     */
    @GetMapping("/order/list")
    public TableDataInfo orderList(EtOrder etOrder)
    {
        startPage();
        List<EtOrder> list = etOrderService.selectEtOrderList(etOrder);
        return getDataTable(list);
    }

    /**
     * 订单详情
     */
    @GetMapping("/order/{orderNo}")
    public AjaxResult orderInfo(@PathVariable("orderNo") String orderNo)
    {
        logger.info("【订单详情】：{}", orderNo);
        return success(etOrderService.getOrderInfo(orderNo));
    }

    /**
     * 改价
     */
    @Log(title = "订单改价", businessType = BusinessType.EDITPRICE)
    @PutMapping("/order/editPrice")
    public AjaxResult orderEdit(@RequestBody EtOrder etOrder)
    {
        logger.info("【订单改价信息】：{}", JSON.toJSON(etOrder));
        //根据订单号查询订单信息
        EtOrder etOrder1 = etOrderService.selectEtOrderByOrderNo(etOrder.getOrderNo());
        if(ObjectUtil.isNull(etOrder1)){
            throw new ServiceException("订单不存在");
        }
        if(!ServiceConstants.ORDER_TYPE_RIDING.equals(etOrder1.getType())){
            throw new ServiceException("改价失败，类型必须是骑行订单");
        }
        if(!ServiceConstants.ORDER_STATUS_RIDING_END.equals(etOrder1.getStatus()) && !ServiceConstants.ORDER_STATUS_CANCEL_APPOINTMENT.equals(etOrder1.getStatus())){
            throw new ServiceException("改价失败，订单未结束，订单状态："+etOrder1.getStatus());
        }
        if(!ServiceConstants.ORDER_PAY_STATUS_NON_PAYMENT.equals(etOrder1.getPaid())){
            throw new ServiceException("订单已支付，不能改价");
        }
        return toAjax(etOrderService.updateEtOrderByOrderNo(etOrder));
    }

    /**
     * 退款
     */
    @Log(title = "订单退款", businessType = BusinessType.REFUND)
    @PutMapping("/order/refund")
    public AjaxResult orderRefund(@RequestBody EtOrder etOrder)
    {
        logger.info("【订单退款请求】：{}", JSON.toJSON(etOrder));
        return toAjax(etOrderService.refund(etOrder));
    }

    /**
     * 提现
     */
    @Transactional
    @Log(title = "提现", businessType = BusinessType.WITHDRAW)
    @PostMapping("/order/withdraw")
    public AjaxResult withdraw()
    {
        //根据订单号查询订单信息
        EtOrder etOrder = new EtOrder();
        etOrder.setUserId(getUserId());
        etOrder.setType(ServiceConstants.ORDER_TYPE_DEPOSIT);
        etOrder.setStatus(ServiceConstants.ORDER_STATUS_ORDER_END);
        etOrder.setPaid(ServiceConstants.ORDER_PAY_STATUS_PAID);
        List<EtOrder> etOrders = etOrderService.selectEtOrderList(etOrder);
        if(etOrders.size()==0){
            throw new ServiceException("提现失败，未找到押金订单");
        }
        if(etOrders.size()>1){
            throw new ServiceException("提现失败，有多条status="+ServiceConstants.ORDER_STATUS_ORDER_END+"（已完成）,paid="+ServiceConstants.ORDER_PAY_STATUS_PAID+"（已支付）的押金订单");
        }
        List<EtOrder> inOrder = etOrderService.isInOrder(etOrder.getUserId(), null);
        if(inOrder.size()>0){
            throw new ServiceException("提现失败，用户还有未完成订单");
        }
        EtOrder etOrder1 = etOrders.get(0);
        EtOperatingArea area = etOperatingAreaService.selectEtOperatingAreaByAreaId(etOrder1.getAreaId());
        BigDecimal deposit = new BigDecimal(area.getDeposit());
        if(deposit.compareTo(etOrder1.getTotalFee())!=0){
            throw new ServiceException("订单充值金额与系统押金不相等");
        }
        Long userId = etOrder1.getUserId();
        if(userId!=null){
            AsUser asUser = asUserService.selectUserById(userId);
            if(asUser!=null){
                BigDecimal result = asUser.getBalance().subtract(deposit);
                int comparisonResult = result.compareTo(BigDecimal.ZERO);
                if (comparisonResult < 0) {
                    throw new ServiceException("余额不足，扣除押金后余额小于0");
                }
                asUser.setBalance(result);//更新用余额
                // 更新用户并更新缓存
                LoginUser loginUser = SecurityUtils.getLoginUser();
                AsUser currentUser = loginUser.getAsUser();
                currentUser.setBalance(result);
                if (asUserService.updateUserProfile(currentUser) > 0)
                {
                    logger.info("【提现金额】更新用户缓存:"+ JSON.toJSON(currentUser));
                    // 更新缓存用户信息
                    tokenService.setLoginUser(loginUser);
                }
            }
        }
        logger.info("提现金额：【{}】", etOrder1.getTotalFee());
        return toAjax(etOrderService.withdraw(etOrder1,deposit));
    }

    /**
     * 车辆预约
     */
    @Log(title = "车辆预约", businessType = BusinessType.APPOINTMENT)
    @PostMapping("/device/appointment")
    public AjaxResult deviceAppointment(EtOrderVo appointmentVo)
    {
        logger.info("【车辆预约信息】：{}", JSON.toJSON(appointmentVo));
        //运营时间判断
        if(!asDeviceService.isOperatingTime(appointmentVo.getSn())){
            return error("不在营业时间内，不得骑行");
        }
        if(etOrderService.isInOrder(appointmentVo.getUserId(),appointmentVo.getOrderNo()).size()>0){
            throw new ServiceException("您有未完成的订单，请先完成订单");
        }
        //根据余额和充值记录判断是否有充值过押金，没有充值过押金，提示充值押金
        if(!asUserService.checkIsDeposit(appointmentVo.getUserId())){
            throw new ServiceException("您还未充值押金，请先充值押金");
        }
        //检验预约订单只能有一个
        if(etOrderService.checkIsUnique(appointmentVo.getUserId())){
            throw new ServiceException("您有预约中的订单，请务重复操作");
        }
        appointmentVo.setType("2");
        AsDevice asDevice = asDeviceService.selectAsDeviceBySn(appointmentVo.getSn());
        String status = asDevice.getStatus();
        if(!ServiceConstants.VEHICLE_STATUS_NORMAL.equals(status)){
            //根据状态值返回不同的提示
            return error(CommonUtil.format(status));
        }
        OrderResponse orderResponse = asDeviceService.deviceAppointment(appointmentVo);
        return success(orderResponse);
    }


    /**
     * 取消预约接口  用户，订单号，sn, ruleId有则是套餐，没有则是普通预约
     */
    @Log(title = "取消预约接口", businessType = BusinessType.CANCEL_APPOINTMENT)
    @PostMapping("/device/cancelAppointment")
    public AjaxResult cancelAppointment(EtOrderVo appointmentVo)
    {
        logger.info("【取消预约接口】：{}", JSON.toJSON(appointmentVo));
        EtOrder order = etOrderService.selectEtOrderByOrderNo(appointmentVo.getOrderNo());
        if(ObjectUtil.isNull(order)){
            return error("没有查询到订单信息："+appointmentVo.getOrderNo());
        }
        AsDevice asDevice = asDeviceService.selectAsDeviceBySn(order.getSn());
        String status = asDevice.getStatus();
        if(!ServiceConstants.VEHICLE_STATUS_IN_APPOINTMENT.equals(status)){
            return error("车辆状态不是预约中");
        }
        OrderResponse orderResponse = asDeviceService.cancelAppointment(appointmentVo,asDevice);
        return success(orderResponse);
    }


    /**
     * 还车
     * 1. 记录还车时间
     * 2. 计算订单各种费用，保存订单总金额
     * 3. 车辆远程关锁
     */
    @Log(title = "还车", businessType = BusinessType.RETURN_VEHICLE)
    @PostMapping("/device/return")
    public AjaxResult returnVehicle(String orderNo,String returnType)
    {
        logger.info("【接收还车请求参数】：{},是否辅助还车：{}", orderNo,returnType);
        Boolean aBoolean = asDeviceService.returnVehicle(orderNo,returnType);
        return success(aBoolean);
    }

    /**
     * 预下单
     * 类型 1骑行 2预约 3套餐 4押金
     */
    @PostMapping("/pre/order")
    public AjaxResult preOrder(@RequestBody EtOrderVo order)
    {
        logger.info("【预下单订单信息】：{}", JSON.toJSON(order));
        PrepayWithRequestPaymentResponse paymentResponse = etOrderService.preOrder(order);
        return success(paymentResponse);
    }

    /**
     * 临时锁车
     */
    @Log(title = "临时锁车", businessType = BusinessType.LOCK)
    @PostMapping("/device/lock")
    public AjaxResult deviceLock(String sn,String orderNo)
    {
        logger.info("【临时锁车请求参数】：{},订单号：{}", sn,orderNo);
        Boolean i =asDeviceService.lock(sn,orderNo);
        return success(i);
    }

    /**
     * 临时解锁
     */
    @Log(title = "临时解锁", businessType = BusinessType.UNLOCK)
    @PostMapping("/device/unlock")
    public AjaxResult deviceUnlock(String sn,String orderNo)
    {
        logger.info("【临时解锁请求参数】：{},订单号：{}", sn,orderNo);
        Boolean i =asDeviceService.unlock(sn,orderNo);
        return success(i);
    }

    /**
     * 更新车牌号
     */
    @Log(title = "更新车牌号", businessType = BusinessType.UPDATE)
    @PutMapping("/device/edit")
    public AjaxResult deviceEdit(String sn, String vehicleNum)
    {
        AsDevice asDevice = new AsDevice();
        asDevice.setSn(sn);
        asDevice.setVehicleNum(vehicleNum);
        return toAjax(asDeviceService.updateAsDeviceBySn(asDevice));
    }


    /**
     * 我的工单列表
     */
    @GetMapping("/adminOrder/list")
    public TableDataInfo adminAsDevice(EtAdminOrder etAdminOrder)
    {
        logger.info("【我的工单列表请求参数】：{}", JSON.toJSON(etAdminOrder));
        startPage();
        List<EtAdminOrder> list = etAdminOrderService.selectEtAdminOrderList(etAdminOrder);
        return getDataTable(list);
    }

    /**
     * 我的工单数
     */
    @GetMapping("/adminOrder/num")
    public AjaxResult adminOrderNum(EtAdminOrder etAdminOrder)
    {
        logger.info("【我的工单数请求参数】：{}", JSON.toJSON(etAdminOrder));
        AdminOrderVo adminOrderVo =etAdminOrderService.selectEtAdminOrderNum(etAdminOrder);
        return success(adminOrderVo);
    }

    /**
     * 报修单详情
     */
    @GetMapping("/adminOrder/{id}")
    public AjaxResult getInfo(@PathVariable("id") Long id)
    {
        logger.info("【报修单详情请求参数】：{}", id);
        return success(etAdminOrderService.selectEtAdminOrderById(id));
    }

    /**
     * 管理员开锁
     */
    @Log(title = "管理员开锁", businessType = BusinessType.UNLOCKING)
    @PostMapping("/admin/unlocking")
    public AjaxResult unlocking(String sn)
    {
        logger.info("【管理员开锁请求】：{}",sn);
        Boolean aBoolean =asDeviceService.unlocking(sn);
        return success(aBoolean);
    }

    /**
     * 管理员关锁
     */
    @Log(title = "管理员关锁", businessType = BusinessType.LOCK)
    @PostMapping("/admin/lock")
    public AjaxResult lock(String sn)
    {
        logger.info("【管理员关锁请求】：{}",sn);
        Boolean aBoolean =asDeviceService.lock(sn,null);
        return success(aBoolean);
    }


    /**
     * 车辆上线
     */
    @Log(title = "车辆上线", businessType = BusinessType.ONLINE)
    @PostMapping("/device/online")
    public AjaxResult online(String sn)
    {
        logger.info("【车辆上线请求】：{}",sn);
        Boolean aBoolean =asDeviceService.online(sn);
        return success(aBoolean);
    }

    /**
     * 车辆下线
     */
    @Log(title = "车辆下线", businessType = BusinessType.OFFLINE)
    @PostMapping("/device/offline")
    public AjaxResult offline(String sn)
    {
        logger.info("【车辆下线请求】：{}",sn);
        Boolean aBoolean =asDeviceService.offline(sn);
        return success(aBoolean);
    }

    /**
     * 处理进度
     */
    @GetMapping("/admiOrder/historyList")
    public TableDataInfo list(EtAdminOrderHistory etAdminOrderHistory)
    {
        startPage();
        List<EtAdminOrderHistory> list = etAdminOrderHistoryService.selectEtAdminOrderHistoryList(etAdminOrderHistory);
        return getDataTable(list);
    }


    /**
     * 维修处理
     */
    @Log(title = "维修处理", businessType = BusinessType.REPAIR)
    @PutMapping("/admiOrder")
    public AjaxResult repairHandle(EtAdminOrder etAdminOrder)
    {
        return toAjax(etAdminOrderService.repairHandle(etAdminOrder));
    }

    /**----------------------------管理员相关----------------------------*/
    /**
     * 所有车辆定位
     */
    @GetMapping(value = "/allVehicleInfo")
    public AjaxResult allVehicleInfo(String powerStart,String powerEnd, String status,String sort)
    {
        List<AsDevice> asDevices = asDeviceService.allVehicleInfo(powerStart,powerEnd,status,sort);
        return success(asDevices);
    }

    /**
     * 批量禁用/解禁
     * disable-禁用 enable-解禁
     */
    @Transactional
    @Log(title = "批量禁用/解禁", businessType = BusinessType.DISABLE)
    @PostMapping("/device/disable")
    public AjaxResult batchDisable(String sns, String disableType)
    {
        logger.info("【批量禁用/解禁请求参数】：{},【{}】", sns, disableType);
        if(StrUtil.isBlank(sns)){
            throw new ServiceException("车辆sn不能为空");
        }
        String[] smList = sns.split(",");
        if(ServiceConstants.BATCH_OPERATION_TYPE_DISABLE.equals(disableType)){//禁用
            for(String sn:smList){
                asDeviceService.offline(sn);
            }
        }else if(ServiceConstants.BATCH_OPERATION_TYPE_ENABLE.equals(disableType)){
            for(String sn:smList){
                asDeviceService.online(sn);
            }
        }else{
            throw new ServiceException("操作类型错误");
        }
        return toAjax(1);
    }

    /**
     * 查询车辆数量
     */
    @GetMapping(value = "/allVehicleNum")
    public AjaxResult allVehicleNum(String powerStart,String powerEnd)
    {
        DeviceNumVo deviceNumVo = asDeviceService.allVehicleNum(powerStart,powerEnd);
        return success(deviceNumVo);
    }

    /**
     * 所有停车点
     */
    @GetMapping(value = "/allParkingArea")
    public AjaxResult allParkingArea()
    {
        List<EtParkingArea> etParkingAreas = etParkingAreaService.allParkingArea();
        return success(etParkingAreas);
    }

    /**
     * 运营数据
     */
    @GetMapping(value = "/operatingData")
    public AjaxResult operatingData(String timeStart,String timeEnd)
    {
        logger.info("【运营数据】请求参数：{},{}", timeStart,timeEnd);
        OperatingDataVo operatingDataVo = etOrderService.getOperatingData(timeStart,timeEnd);
        return success(operatingDataVo);
    }


    /**
     * 收入对账
     * type:1-按日期 2-按车辆
     */
    @GetMapping(value = "/reconciliation")
    public AjaxResult reconciliation(String timeStart,String timeEnd, String type,String sn)
    {
        logger.info("【收入对账】请求参数：timeStart={},timeEnd={},type={},type={}", timeStart,timeEnd,type,type);
        ReconciliationVo operatingDataVo = etOrderService.reconciliation(timeStart,timeEnd,type,sn);
        return success(operatingDataVo);
    }

    /**
     * 最近一笔订单
     */
    @GetMapping(value = "/latestOrder")
    public AjaxResult latestOrder(EtOrder etOrder)
    {
        logger.info("【最近一笔订单】请求参数：etOrder={}", JSON.toJSON(etOrder));
        EtOrder order =etOrderService.latestOrder(etOrder);
        return success(order);
    }


    /**
     * 获取用户列表
     */
    @GetMapping("/user/list")
    public TableDataInfo list(AsUser asUser)
    {
        startPage();
        List<AsUser> list = asUserService.selectUserList(asUser);
        return getDataTable(list);
    }

    /**
     * 分账账单
     *
     * 合伙人账单
     * 合伙人1  分账金额
     */
    @GetMapping("/partnerBill")
    public AjaxResult partnerBill()
    {
//        etOrderService.partnerBill();
        return success();
    }

    /**
     * 实名认证
     */
    @GetMapping("/user/authentication")
    public AjaxResult partnerBillList(AuthenticationVo authenticationVo)
    {
        logger.info("【实名认证】请求参数：authenticationVo={}", JSON.toJSON(authenticationVo));
        Object authentication = asUserService.authentication(authenticationVo);
        //判断o是否是字符串
        if(authentication instanceof String){
            return AjaxResult.error((String) authentication);
        }else{
            IDResponse.Result result = (IDResponse.Result)authentication;
            if(result.getIsok()){
               //保存身份信息
                AsUser asUser = new AsUser();
                asUser.setUserId(authenticationVo.getUserId());
                asUser.setRealName(authenticationVo.getRealName());
                asUser.setIdCard(authenticationVo.getIdCard());
                int updateUser = asUserService.updateUser(asUser);
                if(updateUser==0){
                    throw new ServiceException("【实名认证】保存身份信息失败");
                }else{
                    logger.info("【实名认证】保存身份信息成功");
                }
            }
            return AjaxResult.success(authentication);
        }
    }

    /**
     * 是否缴纳过押金
     */
    @GetMapping(value = "/checkIsDeposit")
    public AjaxResult checkIsDeposit(Long userId)
    {
        Boolean aBoolean = asUserService.checkIsDeposit(userId);
        return AjaxResult.success("操作成功",aBoolean);
    }

}
