package com.ruoyi.web.controller.iot.receive;

import cn.hutool.core.util.ObjectUtil;
import cn.hutool.core.util.StrUtil;
import com.alibaba.fastjson2.JSON;
import com.alibaba.fastjson2.JSONArray;
import com.alibaba.fastjson2.JSONObject;
import com.ruoyi.common.constant.IotConstants;
import com.ruoyi.common.constant.ServiceConstants;
import com.ruoyi.common.core.domain.entity.SysUser;
import com.ruoyi.common.exception.ServiceException;
import com.ruoyi.common.utils.CommonUtil;
import com.ruoyi.common.utils.map.GeoUtils;
import com.ruoyi.common.utils.onenet.Token;
import com.ruoyi.common.utils.uuid.IdUtils;
import com.ruoyi.system.domain.*;
import com.ruoyi.system.service.*;
import com.ruoyi.web.controller.iot.domain.BodyObj;
import com.ruoyi.web.controller.iot.domain.LogEntry;
import com.ruoyi.web.controller.iot.util.Util;
import lombok.SneakyThrows;
import org.locationtech.jts.geom.Geometry;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.beans.factory.annotation.Value;
import org.springframework.transaction.annotation.Transactional;
import org.springframework.web.bind.annotation.*;

import javax.annotation.Resource;
import java.io.UnsupportedEncodingException;
import java.math.BigDecimal;

/**
 * 接收硬件参数
 *
 * @author ruoyi
 */
@RestController
@RequestMapping("/common")
public class ReceiveController {

    private static final Logger log = LoggerFactory.getLogger(ReceiveController.class);

    @Value(value = "${iot.token}")
    private String token;


    @Resource
    private IAsDeviceService asDeviceService;

    @Autowired
    private IEtModelService etModelService;

    @Resource
    private IEtOperatingAreaService etOperatingAreaService;

    @Autowired
    private IEtAdminOrderService etAdminOrderService;

    @Autowired
    private ISysUserService sysUserService;

    @Autowired
    private IEtOrderService etOrderService;


    /**
     * 功能描述：第三方平台数据接收。<p>
     *           <ul>注:
     *               <li>1.OneNet平台为了保证数据不丢失，有重发机制，如果重复数据对业务有影响，数据接收端需要对重复数据进行排除重复处理。</li>
     *               <li>2.OneNet每一次post数据请求后，等待客户端的响应都设有时限，在规定时限内没有收到响应会认为发送失败。
     *                    接收程序接收到数据时，尽量先缓存起来，再做业务逻辑处理。</li>
     *           </ul>
     * @param body 数据消息
     * @return 任意字符串。OneNet平台接收到http 200的响应，才会认为数据推送成功，否则会重发。
     */
    @RequestMapping(value = "/receive",method = RequestMethod.POST)
    @ResponseBody
    @SneakyThrows
    @Transactional
    public String receive(@RequestBody String body){

        log.info("receive方法接收到参数:  body String --- " +body);
        /************************************************
         *  解析数据推送请求，非加密模式。
         *  如果是明文模式使用以下代码
         **************************************************/
        /*************明文模式  start****************/
        BodyObj obj = Util.resolveBody(body, false);
        log.info("receive方法解析对象:  body Object --- " + JSON.toJSONString(obj));
        if (obj != null){
            boolean dataRight = Util.checkSignature(obj, token);
            if (dataRight){
                log.info("receive方法验证签名正确: content" + JSON.toJSONString(obj));
                String msg = (String)obj.getMsg();
                log.info("receive方法-获取到消息体: msg---" +msg);
                LogEntry logEntry = JSONObject.parseObject(msg, LogEntry.class);
                log.info("logEntry转换后的对象: logEntry---【{}】" , JSON.toJSONString(logEntry));
                LogEntry.LocationValue value = logEntry.getValue();
                if(IotConstants.ONENET_LOCATION.equals(logEntry.getDsId()) && ObjectUtil.isNotNull(value)){
                    /**如果是定位日志则，获取到车辆mac，找到对应车辆
                     * 1.更新车辆定位、计算续航里程
                     * 2.判断是否在禁行区内，如果在，根据配置‘禁行区内断电配置’进行断电，
                     * 3.超出运营区外断电
                     * 4.行程线路添加，更新订单中的trip_route字段
                     * 5.低于电量（%）不得骑行，声音播报
                     * 6.低电量 生成换电工单
                     * 7.运营边界判断
                     * */
                    /** 1.更新车辆定位、电压；计算续航里程 */
                    AsDevice device = asDeviceService.selectAsDeviceByMac(logEntry.getDevName());
                    if(ObjectUtil.isNull(device)){
                        throw new ServiceException("未找到车辆信息");
                    }
                    EtOperatingArea area = etOperatingAreaService.selectEtOperatingAreaByAreaId(device.getAreaId());
                    if(ObjectUtil.isNotNull(device)){
                        device.setLatitude(value.getLon());
                        device.setLongitude(value.getLat());
                        Integer bat = value.getBat();
                        BigDecimal divide = new BigDecimal(bat).divide(new BigDecimal(10));
                        log.info("保存电压：" + divide);
                        device.setVoltage(divide.toString());//电压
                        // 根据电压计算续航里程
                        EtModel model = etModelService.selectEtModelByModelId(device.getModelId());
                        Integer remainingMileage = CommonUtil.getRemainingMileage(device.getVoltage(), model.getFullVoltage(), model.getLowVoltage(), model.getFullEndurance());
                        Integer electricQuantity = CommonUtil.getElectricQuantity(device.getVoltage(), model.getFullVoltage(), model.getLowVoltage());//电量百分百
                        device.setRemainingMileage(remainingMileage);
                        device.setRemainingPower(electricQuantity.toString());
                        int i = asDeviceService.updateAsDeviceBySn(device);
                        if(i>0){
                            log.info("更新定位成功==========================>" +logEntry.getDevName());
                            /** 2. 判断是否在禁行区内
                             *      如果在， 根据配置‘禁行区内断电配置’进行断电
                             **/
                            boolean noRidingArea = asDeviceService.isNoRidingArea(device.getSn(), device.getAreaId());
                            if(noRidingArea){
                                String noRidingOutage = area.getNoRidingOutage();
                                //发送播报指令
                                asDeviceService.sendCommand(device.getMac(), Token.getToken(),IotConstants.COMMAND_PLAY3,"禁行区内播报");
                                if(noRidingOutage.equals("1")){//禁行区内断电
                                    //发送断电命令
                                    asDeviceService.sendCommand(device.getMac(), Token.getToken(),IotConstants.COMMAND_QLOSE,"禁行区内断电");
                                    log.info("禁行区内发送断电命令：" +logEntry.getDevName());
                                }
                            }
                            /** 3.超出运营区外断电*/
                            boolean isAreaZone = asDeviceService.isAreaZone(device.getSn(), area);
                            if(isAreaZone){
                                String areaOutOutage = area.getAreaOutOutage();
                                //发送超出营运区播报指令
                                asDeviceService.sendCommand(device.getMac(), Token.getToken(),IotConstants.COMMAND_PLAY3,"超出营运区播报");
                                if(areaOutOutage.equals("1")){//超出营运区断电
                                    //发送断电命令
                                    asDeviceService.sendCommand(device.getMac(), Token.getToken(),IotConstants.COMMAND_QLOSE,"超出营运区断电");
                                    log.info("超出营运区发送断电命令：" +logEntry.getDevName());
                                }
                            }
                            /** TODO 4.行程线路添加，更新订单中的trip_route字段 */
                            Integer status = value.getStatus();
                            if(status == 1){//上电运行
                                log.info("上电运行：" +logEntry.getDevName());
                                //获取当前正在骑行中的订单
                                EtOrder etOrder = etOrderService.getCurrentOrder(device.getSn());
                                JSONArray jsonArray;
                                if(StrUtil.isNotBlank(etOrder.getTripRouteStr())){
                                    jsonArray = JSON.parseArray(etOrder.getTripRouteStr());
                                }else{
                                    jsonArray = new JSONArray();
                                }
                                JSONArray newPoint = new JSONArray();
                                newPoint.add(value.getLon());
                                newPoint.add(value.getLat());
                                jsonArray.add(newPoint);
                                jsonArray.toJSONString();

                                log.info("更新行程jsonArray：" +jsonArray.toJSONString());
                                etOrder.setTripRouteStr(jsonArray.toJSONString());
                                Geometry geometry = GeoUtils.toGeometry(jsonArray.toJSONString());
                                String wkt = GeoUtils.wkt(geometry);
                                etOrder.setTripRoute(wkt);
                                int updateEtOrder = etOrderService.updateEtOrder(etOrder);
                                if (updateEtOrder > 0) {
                                    log.info("更新行程线路成功==========================>" +logEntry.getDevName());
                                }else{
                                    log.info("更新行程线路失败==========================>" +logEntry.getDevName());
                                }
                            }

                            /** 5.低于电量（%）不得骑行，声音播报 */
                            if(electricQuantity <= model.getLowBatteryReminder()){
                                //发送低电量播报指令
//                                asDeviceService.sendCommand(device.getMac(), Token.getToken(),IotConstants.COMMAND_PLAY6,"低电量播报");
                                log.info("低电量播报：" +logEntry.getDevName());
                            }
                            /** 6.低电量 生成换电工单*/
                            Integer replacementOrder = area.getAutoReplacementOrder();
                            Boolean aBoolean = etAdminOrderService.checkOrderUnique(device.getSn());
                            if (Integer.parseInt(device.getRemainingPower()) < replacementOrder && !aBoolean) {
                                /** 生成换电工单 */
                                EtAdminOrder adminOrder = new EtAdminOrder();
                                adminOrder.setDeviceMac(device.getMac());
                                adminOrder.setSn(device.getSn());
                                adminOrder.setBeforeElectric(Integer.parseInt(device.getRemainingPower()));
                                adminOrder.setOrderNo(IdUtils.randomUUID2());
                                adminOrder.setStatus(ServiceConstants.REPLACEMENT_ELECTRICITY_STATUS_UNDER_WAY);
                                adminOrder.setIsEffective(ServiceConstants.REPLACEMENT_ELECTRICITY_IS_EFFECTIVE_YES);
                                SysUser sysUser = sysUserService.selectUserByUserName("wx");//获取用户名为“wx”的维修人员
                                adminOrder.setAdminId(sysUser.getUserId());
                                adminOrder.setAdminName(sysUser.getUserName());
                                adminOrder.setLatitude(device.getLatitude());
                                adminOrder.setLongitude(device.getLongitude());
                                adminOrder.setType("2");
                                String location = device.getLongitude() + ","+device.getLatitude();
                                adminOrder.setAddress(CommonUtil.getAddressByGeo(location));
                                int i1 = etAdminOrderService.insertEtAdminOrder(adminOrder);
                                if (i1 > 0) {
                                    log.info("生成换电工单成功");
                                }else{
                                    throw new ServiceException("生成换电工单失败");
                                }
                                /** 改变车辆状态 */
                                device.setStatus(ServiceConstants.VEHICLE_STATUS_IN_OFFLINE);
                                if (asDeviceService.updateAsDevice(device) > 0) {
                                    log.info("车辆状态改成换电中");
                                }else{
                                    throw new ServiceException("车辆状态更新失败");
                                }
                            }
//                            /** TODO 7.运营边界判断  几米判断？ 3  播报运营边界  play2@  是否靠近运营区边界*/
//                            boolean isCloseToTheBoundary = asDeviceService.isCloseToTheBoundary(device.getSn(), area);
//                            if(isCloseToTheBoundary){
//                                //发送超出营运区播报指令
//                                asDeviceService.sendCommand(device.getMac(), Token.getToken(),IotConstants.COMMAND_PLAY2,"靠近营运边界播报");
//                            }

                        }else{
                            log.info("更新定位失败：" +logEntry.getDevName());
                        }
                    }else{
                        log.info("未找到车辆对象：" +logEntry.getDevName());
                    }

                }
            }else {
                log.info("receive方法验证签名错误: signature error");
            }

        }else {
            log.info("receive方法参数为空: body empty error");
        }
        /*************明文模式  end****************/
        return "ok";
    }

    /**
     * 功能说明： URL&Token验证接口。如果验证成功返回msg的值，否则返回其他值。
     * @param msg 验证消息
     * @param nonce 随机串
     * @param signature 签名
     * @return msg值
     */

    @RequestMapping(value = "/receive", method = RequestMethod.GET)
    @ResponseBody
    public String check(@RequestParam(value = "msg") String msg,
                        @RequestParam(value = "nonce") String nonce,
                        @RequestParam(value = "signature") String signature) throws UnsupportedEncodingException {

        log.info("check方法接收到参数：: msg:{} nonce{} signature:{}",msg,nonce,signature);
        if (Util.checkToken(msg,nonce,signature,token)){
            log.info("校验成功",msg,nonce,signature);
            return msg;
        }else {
            log.info("校验失败",msg,nonce,signature);
            return "error";
        }

    }
}
