package com.ruoyi.web.controller.system;

import com.ruoyi.common.annotation.Log;
import com.ruoyi.common.constant.UserConstants;
import com.ruoyi.common.core.controller.BaseController;
import com.ruoyi.common.core.domain.AjaxResult;
import com.ruoyi.common.enums.BusinessType;
import com.ruoyi.common.utils.StringUtils;
import com.ruoyi.common.core.domain.entity.AsArticleClassify;
import com.ruoyi.system.service.IAsArticleClassifyService;
import org.apache.commons.lang3.ArrayUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.validation.annotation.Validated;
import org.springframework.web.bind.annotation.*;

import java.util.List;

/**
 * 分类信息
 *
 * @author ruoyi
 */
@RestController
@RequestMapping("/article/classify")
public class AsArticleClassifyController extends BaseController
{
    @Autowired
    private IAsArticleClassifyService asArticleClassifyService;

    /**
     * 获取分类列表
     */
    @GetMapping("/list")
    public AjaxResult list(AsArticleClassify asArticleClassify)
    {
        List<AsArticleClassify> asArticleClassifies = asArticleClassifyService.selectClassifyList(asArticleClassify);
        return success(asArticleClassifies);
    }

    /**
     * 查询分类列表（排除节点）
     */
    @GetMapping("/list/exclude/{classifyId}")
    public AjaxResult excludeChild(@PathVariable(value = "classifyId", required = false) Long classifyId)
    {
        List<AsArticleClassify> articleClassifies = asArticleClassifyService.selectClassifyList(new AsArticleClassify());
        articleClassifies.removeIf(d -> d.getClassifyId().intValue() == classifyId || ArrayUtils.contains(StringUtils.split(d.getAncestors(), ","), classifyId + ""));
        return success(articleClassifies);
    }

    /**
     * 根据分类编号获取详细信息
     */
    @GetMapping(value = "/{classifyId}")
    public AjaxResult getInfo(@PathVariable Long classifyId)
    {
        asArticleClassifyService.checkClassifyDataScope(classifyId);
        return success(asArticleClassifyService.selectClassifyById(classifyId));
    }

    /**
     * 新增分类
     */
    @Log(title = "分类管理", businessType = BusinessType.INSERT)
    @PostMapping
    public AjaxResult add(@Validated @RequestBody AsArticleClassify dept)
    {
        if (!asArticleClassifyService.checkClassifyNameUnique(dept))
        {
            return error("新增分类'" + dept.getClassifyName() + "'失败，分类名称已存在");
        }
        dept.setCreateBy(getUsername());
        return toAjax(asArticleClassifyService.insertClassify(dept));
    }

    /**
     * 修改分类
     */
    @Log(title = "分类管理", businessType = BusinessType.UPDATE)
    @PutMapping
    public AjaxResult edit(@Validated @RequestBody AsArticleClassify articleClassify)
    {
        Long classifyId = articleClassify.getClassifyId();
        asArticleClassifyService.checkClassifyDataScope(classifyId);
        if (!asArticleClassifyService.checkClassifyNameUnique(articleClassify))
        {
            return error("修改分类'" + articleClassify.getClassifyName() + "'失败，分类名称已存在");
        }
        else if (articleClassify.getParentId().equals(classifyId))
        {
            return error("修改分类'" + articleClassify.getClassifyName() + "'失败，上级分类不能是自己");
        }
        else if (StringUtils.equals(UserConstants.DEPT_DISABLE, articleClassify.getStatus()) && asArticleClassifyService.selectNormalChildrenClassifyById(classifyId) > 0)
        {
            return error("该分类包含未停用的子分类！");
        }
        articleClassify.setUpdateBy(getUsername());
        return toAjax(asArticleClassifyService.updateClassify(articleClassify));
    }

    /**
     * 删除分类
     */
    @Log(title = "分类管理", businessType = BusinessType.DELETE)
    @DeleteMapping("/{classifyId}")
    public AjaxResult remove(@PathVariable Long classifyId)
    {
        if (asArticleClassifyService.hasChildByclassifyId(classifyId))
        {
            return warn("存在下级分类,不允许删除");
        }
        if (asArticleClassifyService.checkClassifyExistArticle(classifyId))
        {
            return warn("分类存在文章,不允许删除");
        }
        asArticleClassifyService.checkClassifyDataScope(classifyId);
        return toAjax(asArticleClassifyService.deleteClassifyById(classifyId));
    }
}
