package com.ruoyi.web.controller.system;

import com.ruoyi.common.annotation.Log;
import com.ruoyi.common.core.controller.BaseController;
import com.ruoyi.common.core.domain.AjaxResult;
import com.ruoyi.common.core.domain.entity.AsArticleClassify;
import com.ruoyi.common.core.page.TableDataInfo;
import com.ruoyi.common.enums.BusinessType;
import com.ruoyi.common.utils.poi.ExcelUtil;
import com.ruoyi.system.domain.EtArticle;
import com.ruoyi.system.service.IAsArticleClassifyService;
import com.ruoyi.system.service.IAsArticleService;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.security.access.prepost.PreAuthorize;
import org.springframework.web.bind.annotation.*;

import javax.servlet.http.HttpServletResponse;
import java.util.List;

/**
 * 文章Controller
 *
 * @author qiuzhenzhao
 * @date 2023-12-06
 */
@RestController
@RequestMapping("/admin/article")
public class AsArticleController extends BaseController
{
    @Autowired
    private IAsArticleService asArticleService;

    @Autowired
    private IAsArticleClassifyService asArticleClassifyService;

    /**
     * 查询文章列表
     */
//    @PreAuthorize("@ss.hasPermi('system:article:list')")
    @GetMapping("/list")
    public TableDataInfo list(EtArticle etArticle)
    {
        startPage();
        List<EtArticle> list = asArticleService.selectAsArticleList(etArticle);
        return getDataTable(list);
    }

    /**
     * 导出文章列表
     */
//    @PreAuthorize("@ss.hasPermi('system:article:export')")
    @Log(title = "文章", businessType = BusinessType.EXPORT)
    @PostMapping("/export")
    public void export(HttpServletResponse response, EtArticle etArticle)
    {
        List<EtArticle> list = asArticleService.selectAsArticleList(etArticle);
        ExcelUtil<EtArticle> util = new ExcelUtil<EtArticle>(EtArticle.class);
        util.exportExcel(response, list, "文章数据");
    }

    /**
     * 获取文章详细信息
     */
//    @PreAuthorize("@ss.hasPermi('system:article:query')")
    @GetMapping(value = "/{articleId}")
    public AjaxResult getInfo(@PathVariable("articleId") Long articleId)
    {
        return success(asArticleService.selectAsArticleByArticleId(articleId));
    }

    /**
     * 新增文章
     */
//    @PreAuthorize("@ss.hasPermi('system:article:add')")
    @Log(title = "文章", businessType = BusinessType.INSERT)
    @PostMapping
    public AjaxResult add(@RequestBody EtArticle etArticle)
    {
        return toAjax(asArticleService.insertAsArticle(etArticle));
    }

    /**
     * 修改文章
     */
//    @PreAuthorize("@ss.hasPermi('system:article:edit')")
    @Log(title = "文章", businessType = BusinessType.UPDATE)
    @PutMapping
    public AjaxResult edit(@RequestBody EtArticle etArticle)
    {
        return toAjax(asArticleService.updateAsArticle(etArticle));
    }

    /**
     * 删除文章
     */
//    @PreAuthorize("@ss.hasPermi('system:article:remove')")
    @Log(title = "文章", businessType = BusinessType.DELETE)
	@DeleteMapping("/{articleIds}")
    public AjaxResult remove(@PathVariable Long[] articleIds)
    {
        return toAjax(asArticleService.deleteAsArticleByArticleIds(articleIds));
    }

    /**
     * 获取分类树列表
     */
    @PreAuthorize("@ss.hasPermi('system:user:list')")
    @GetMapping("/classifyTree")
    public AjaxResult deptTree(AsArticleClassify classify)
    {
        return success(asArticleClassifyService.selectClassifyTreeList(classify));
    }
}
