package com.ruoyi.web.controller.system;

import com.alibaba.fastjson2.JSON;
import com.ruoyi.common.annotation.Log;
import com.ruoyi.common.core.controller.BaseController;
import com.ruoyi.common.core.domain.AjaxResult;
import com.ruoyi.common.core.page.TableDataInfo;
import com.ruoyi.common.enums.BusinessType;
import com.ruoyi.common.utils.poi.ExcelUtil;
import com.ruoyi.system.domain.EtOperatingArea;
import com.ruoyi.system.service.IEtFeeRuleService;
import com.ruoyi.system.service.IEtOperatingAreaService;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.security.access.prepost.PreAuthorize;
import org.springframework.web.bind.annotation.*;

import javax.servlet.http.HttpServletResponse;
import java.util.List;

/**
 * 运营区Controller
 *
 * @author 邱贞招
 * @date 2024-04-06
 */
@RestController
@RequestMapping("/system/area")
public class EtOperatingAreaController extends BaseController
{
    @Autowired
    private IEtOperatingAreaService etOperatingAreaService;

    @Autowired
    private IEtFeeRuleService etFeeRuleService;

    /**
     * 查询运营区列表
     */
    @PreAuthorize("@ss.hasPermi('system:area:list')")
    @GetMapping("/list")
    public TableDataInfo list(EtOperatingArea etOperatingArea)
    {
        startPage();
        List<EtOperatingArea> list = etOperatingAreaService.selectEtOperatingAreaList(etOperatingArea);
        return getDataTable(list);
    }

    /**
     * 导出运营区列表
     */
    @PreAuthorize("@ss.hasPermi('system:area:export')")
    @Log(title = "运营区", businessType = BusinessType.EXPORT)
    @PostMapping("/export")
    public void export(HttpServletResponse response, EtOperatingArea etOperatingArea)
    {
        List<EtOperatingArea> list = etOperatingAreaService.selectEtOperatingAreaList(etOperatingArea);
        ExcelUtil<EtOperatingArea> util = new ExcelUtil<EtOperatingArea>(EtOperatingArea.class);
        util.exportExcel(response, list, "运营区数据");
    }

    /**
     * 获取运营区详细信息
     */
    @PreAuthorize("@ss.hasPermi('system:area:query')")
    @GetMapping(value = "/{areaId}")
    public AjaxResult getInfo(@PathVariable("areaId") Long areaId)
    {
        AjaxResult ajax = AjaxResult.success();
        EtOperatingArea area = etOperatingAreaService.selectEtOperatingAreaByAreaId(areaId);
        ajax.put(AjaxResult.DATA_TAG, area);
        ajax.put("ruleIds", etFeeRuleService.selectRuleListByAreaId(areaId));
        return ajax;
    }

    /**
     * 新增运营区
     */
    @PreAuthorize("@ss.hasPermi('system:area:add')")
    @Log(title = "运营区", businessType = BusinessType.INSERT)
    @PostMapping
    public AjaxResult add(@RequestBody EtOperatingArea etOperatingArea)
    {
        logger.info("新增运营区:{}",JSON.toJSONString(etOperatingArea));
        if (!etOperatingAreaService.checkAreaNameUnique(etOperatingArea))
        {
            return error("新增运营区'" + etOperatingArea.getAreaName() + "'失败，该运营区已存在");
        }
        etOperatingArea.setCreateBy(getUsername());
        return toAjax(etOperatingAreaService.insertEtOperatingArea(etOperatingArea));
    }

    /**
     * 修改运营区
     */
    @PreAuthorize("@ss.hasPermi('system:area:edit')")
    @Log(title = "运营区", businessType = BusinessType.UPDATE)
    @PutMapping
    public AjaxResult edit(@RequestBody EtOperatingArea etOperatingArea)
    {
        if (!etOperatingAreaService.checkAreaNameUnique(etOperatingArea))
        {
            return error("修改运营区'" + etOperatingArea.getAreaName() + "'失败，该运营区已存在");
        }
        return toAjax(etOperatingAreaService.updateEtOperatingArea(etOperatingArea));
    }

    /**
     * 删除运营区
     */
    @PreAuthorize("@ss.hasPermi('system:area:remove')")
    @Log(title = "运营区", businessType = BusinessType.DELETE)
	@DeleteMapping("/{areaIds}")
    public AjaxResult remove(@PathVariable Long[] areaIds)
    {
        return toAjax(etOperatingAreaService.deleteEtOperatingAreaByAreaIds(areaIds));
    }

    /**
     * 获取分区选择框列表
     */
    @GetMapping("/optionselect")
    public AjaxResult optionselect()
    {
        List<EtOperatingArea> etOperatingAreas = etOperatingAreaService.selectAreaAll();
        return success(etOperatingAreas);
    }
}
