package com.ruoyi.web.controller.system;

import java.util.List;
import javax.servlet.http.HttpServletResponse;
import org.springframework.security.access.prepost.PreAuthorize;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.web.bind.annotation.GetMapping;
import org.springframework.web.bind.annotation.PostMapping;
import org.springframework.web.bind.annotation.PutMapping;
import org.springframework.web.bind.annotation.DeleteMapping;
import org.springframework.web.bind.annotation.PathVariable;
import org.springframework.web.bind.annotation.RequestBody;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RestController;
import com.ruoyi.common.annotation.Log;
import com.ruoyi.common.core.controller.BaseController;
import com.ruoyi.common.core.domain.AjaxResult;
import com.ruoyi.common.enums.BusinessType;
import com.ruoyi.system.domain.EtParkingArea;
import com.ruoyi.system.service.IEtParkingAreaService;
import com.ruoyi.common.utils.poi.ExcelUtil;
import com.ruoyi.common.core.page.TableDataInfo;

/**
 * 停车区Controller
 *
 * @author 邱贞招
 * @date 2024-04-08
 */
@RestController
@RequestMapping("/system/parking")
public class EtParkingAreaController extends BaseController
{
    @Autowired
    private IEtParkingAreaService etParkingAreaService;

    /**
     * 查询停车区列表
     */
    @PreAuthorize("@ss.hasPermi('system:parking:list')")
    @GetMapping("/list")
    public TableDataInfo list(EtParkingArea etParkingArea)
    {
        startPage();
        List<EtParkingArea> list = etParkingAreaService.selectEtParkingAreaList(etParkingArea);
        return getDataTable(list);
    }

    /**
     * 导出停车区列表
     */
    @PreAuthorize("@ss.hasPermi('system:parking:export')")
    @Log(title = "停车区", businessType = BusinessType.EXPORT)
    @PostMapping("/export")
    public void export(HttpServletResponse response, EtParkingArea etParkingArea)
    {
        List<EtParkingArea> list = etParkingAreaService.selectEtParkingAreaList(etParkingArea);
        ExcelUtil<EtParkingArea> util = new ExcelUtil<EtParkingArea>(EtParkingArea.class);
        util.exportExcel(response, list, "停车区数据");
    }

    /**
     * 获取停车区详细信息
     */
    @PreAuthorize("@ss.hasPermi('system:parking:query')")
    @GetMapping(value = "/{parkingId}")
    public AjaxResult getInfo(@PathVariable("parkingId") Long parkingId)
    {
        return success(etParkingAreaService.selectEtParkingAreaByParkingId(parkingId));
    }

    /**
     * 新增停车区
     */
    @PreAuthorize("@ss.hasPermi('system:parking:add')")
    @Log(title = "停车区", businessType = BusinessType.INSERT)
    @PostMapping
    public AjaxResult add(@RequestBody EtParkingArea etParkingArea)
    {
        if (!etParkingAreaService.checkParkingNameUnique(etParkingArea))
        {
            return error("新增停车区'" + etParkingArea.getParkingName() + "'失败，该停车区已存在");
        }
        return toAjax(etParkingAreaService.insertEtParkingArea(etParkingArea));
    }

    /**
     * 修改停车区
     */
    @PreAuthorize("@ss.hasPermi('system:parking:edit')")
    @Log(title = "停车区", businessType = BusinessType.UPDATE)
    @PutMapping
    public AjaxResult edit(@RequestBody EtParkingArea etParkingArea)
    {
        if (!etParkingAreaService.checkParkingNameUnique(etParkingArea))
        {
            return error("修改停车区'" + etParkingArea.getParkingName() + "'失败，该停车区已存在");
        }
        return toAjax(etParkingAreaService.updateEtParkingArea(etParkingArea));
    }

    /**
     * 删除停车区
     */
    @PreAuthorize("@ss.hasPermi('system:parking:remove')")
    @Log(title = "停车区", businessType = BusinessType.DELETE)
	@DeleteMapping("/{parkingIds}")
    public AjaxResult remove(@PathVariable Long[] parkingIds)
    {
        return toAjax(etParkingAreaService.deleteEtParkingAreaByParkingIds(parkingIds));
    }
}
