package com.ruoyi.web.controller.system;

import java.util.List;
import javax.servlet.http.HttpServletResponse;
import org.springframework.security.access.prepost.PreAuthorize;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.web.bind.annotation.GetMapping;
import org.springframework.web.bind.annotation.PostMapping;
import org.springframework.web.bind.annotation.PutMapping;
import org.springframework.web.bind.annotation.DeleteMapping;
import org.springframework.web.bind.annotation.PathVariable;
import org.springframework.web.bind.annotation.RequestBody;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RestController;
import com.ruoyi.common.annotation.Log;
import com.ruoyi.common.core.controller.BaseController;
import com.ruoyi.common.core.domain.AjaxResult;
import com.ruoyi.common.enums.BusinessType;
import com.ruoyi.system.domain.EtRefund;
import com.ruoyi.system.service.IEtRefundService;
import com.ruoyi.common.utils.poi.ExcelUtil;
import com.ruoyi.common.core.page.TableDataInfo;

/**
 * 退款订单Controller
 *
 * @author 邱贞招
 * @date 2024-05-10
 */
@RestController
@RequestMapping("/system/refund")
public class EtRefundController extends BaseController
{
    @Autowired
    private IEtRefundService etRefundService;

    /**
     * 查询退款订单列表
     */
    @PreAuthorize("@ss.hasPermi('system:refund:list')")
    @GetMapping("/list")
    public TableDataInfo list(EtRefund etRefund)
    {
        startPage();
        List<EtRefund> list = etRefundService.selectEtRefundList(etRefund);
        return getDataTable(list);
    }

    /**
     * 导出退款订单列表
     */
    @PreAuthorize("@ss.hasPermi('system:refund:export')")
    @Log(title = "退款订单", businessType = BusinessType.EXPORT)
    @PostMapping("/export")
    public void export(HttpServletResponse response, EtRefund etRefund)
    {
        List<EtRefund> list = etRefundService.selectEtRefundList(etRefund);
        ExcelUtil<EtRefund> util = new ExcelUtil<EtRefund>(EtRefund.class);
        util.exportExcel(response, list, "退款订单数据");
    }

    /**
     * 获取退款订单详细信息
     */
    @PreAuthorize("@ss.hasPermi('system:refund:query')")
    @GetMapping(value = "/{id}")
    public AjaxResult getInfo(@PathVariable("id") Long id)
    {
        return success(etRefundService.selectEtRefundById(id));
    }

    /**
     * 新增退款订单
     */
    @PreAuthorize("@ss.hasPermi('system:refund:add')")
    @Log(title = "退款订单", businessType = BusinessType.INSERT)
    @PostMapping
    public AjaxResult add(@RequestBody EtRefund etRefund)
    {
        return toAjax(etRefundService.insertEtRefund(etRefund));
    }

    /**
     * 修改退款订单
     */
    @PreAuthorize("@ss.hasPermi('system:refund:edit')")
    @Log(title = "退款订单", businessType = BusinessType.UPDATE)
    @PutMapping
    public AjaxResult edit(@RequestBody EtRefund etRefund)
    {
        return toAjax(etRefundService.updateEtRefund(etRefund));
    }

    /**
     * 删除退款订单
     */
    @PreAuthorize("@ss.hasPermi('system:refund:remove')")
    @Log(title = "退款订单", businessType = BusinessType.DELETE)
	@DeleteMapping("/{ids}")
    public AjaxResult remove(@PathVariable Long[] ids)
    {
        return toAjax(etRefundService.deleteEtRefundByIds(ids));
    }
}
