package com.ruoyi.web.controller.system;

import java.util.List;
import java.util.Set;

import cn.hutool.core.util.ObjectUtil;
import com.ruoyi.common.core.domain.entity.AsUser;
import com.ruoyi.system.service.IAsUserService;
import com.ruoyi.system.service.ISysUserService;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.web.bind.annotation.GetMapping;
import org.springframework.web.bind.annotation.PostMapping;
import org.springframework.web.bind.annotation.RequestBody;
import org.springframework.web.bind.annotation.RestController;
import com.ruoyi.common.constant.Constants;
import com.ruoyi.common.core.domain.AjaxResult;
import com.ruoyi.common.core.domain.entity.SysMenu;
import com.ruoyi.common.core.domain.entity.SysUser;
import com.ruoyi.common.core.domain.model.LoginBody;
import com.ruoyi.common.utils.SecurityUtils;
import com.ruoyi.framework.web.service.SysLoginService;
import com.ruoyi.framework.web.service.SysPermissionService;
import com.ruoyi.system.service.ISysMenuService;

/**
 * 登录验证
 *
 * @author ruoyi
 */
@RestController
public class SysLoginController
{
    @Autowired
    private SysLoginService loginService;

    @Autowired
    private ISysMenuService menuService;

    @Autowired
    private SysPermissionService permissionService;

    @Autowired
    private IAsUserService userService;


    /**
     * 登录方法
     *
     * @param loginBody 登录信息
     * @return 结果
     */
    @PostMapping("/login")
    public AjaxResult login(@RequestBody LoginBody loginBody)
    {
        AjaxResult ajax = AjaxResult.success();
        // 生成令牌
        String token = loginService.login(loginBody.getUsername(), loginBody.getPassword(), loginBody.getCode(),
                loginBody.getUuid());
        ajax.put(Constants.TOKEN, token);
        return ajax;
    }

    /**
     * 获取用户信息
     *
     * @return 用户信息
     */
    @GetMapping("getInfo")
    public AjaxResult getInfo()
    {
        SysUser user = SecurityUtils.getLoginUser().getUser();
        // 角色集合
        Set<String> roles = permissionService.getRolePermission(user);
        // 权限集合
        Set<String> permissions = permissionService.getMenuPermission(user);
        AjaxResult ajax = AjaxResult.success();
        ajax.put("user", user);
        ajax.put("roles", roles);
        ajax.put("permissions", permissions);
        return ajax;
    }

    /**
     * 获取app用户信息
     *
     * @return 用户信息
     */
    @GetMapping("getAppInfo")
    public AjaxResult getAppInfo(Long userId)
    {
        AjaxResult ajax = AjaxResult.success();
        AsUser user;
        if(ObjectUtil.isNotNull(userId)){
            user =  userService.selectUserById(userId);
        }else{
            user = SecurityUtils.getLoginUser().getAsUser();
        }
        ajax.put("user", user);
        return ajax;
    }

    /**
     * 获取路由信息
     *
     * @return 路由信息
     */
    @GetMapping("getRouters")
    public AjaxResult getRouters()
    {
        Long userId = SecurityUtils.getUserId();
        List<SysMenu> menus = menuService.selectMenuTreeByUserId(userId);
        return AjaxResult.success(menuService.buildMenus(menus));
    }

//    @PostMapping("/appCodeLogin")
//    public AjaxResult appCodeLogin(@RequestBody LoginBody loginBody) {
//        AjaxResult ajax = AjaxResult.success();
//        /**通过手机号找到用户名*/
//        String phone = loginBody.getPhone();
//        AsUser user = asUserService.selectUserByPhone(phone);
//        // 生成令牌
//        String token = loginService.appCodeLogin(user.getUserName(), loginBody.getPhoneCode(), loginBody.getUuid());
//        ajax.put(Constants.TOKEN, token);
//        return ajax;
//    }

    /**
     * 微信登录
     */
    @PostMapping("/wxlogin")
    public AjaxResult wxlogin(String mobileCode,String jsCode,String areaId) {
        AjaxResult ajax = AjaxResult.success();
        // 生成令牌
        String token = loginService.wxloing(mobileCode,jsCode,areaId);
        ajax.put(Constants.TOKEN, token);
        return ajax;
    }

    /**
     * 根据openid静默登录
     */
    @PostMapping("/loginByopenid")
    public AjaxResult loginByopenid(String jsCode,String areaId) {
        AjaxResult ajax = AjaxResult.success();
        // 生成令牌
        String token = loginService.loginByopenid(jsCode,areaId);
        ajax.put(Constants.TOKEN, token);
        return ajax;
    }
}
