package com.ruoyi.framework.web.service;

import com.ruoyi.common.constant.Constants;
import com.ruoyi.common.core.domain.entity.AsUser;
import com.ruoyi.framework.security.context.AuthenticationContextHolder;
import com.ruoyi.system.service.IAsUserService;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.security.core.Authentication;
import org.springframework.security.core.userdetails.UserDetails;
import org.springframework.security.core.userdetails.UserDetailsService;
import org.springframework.security.core.userdetails.UsernameNotFoundException;
import org.springframework.stereotype.Service;
import com.ruoyi.common.core.domain.entity.SysUser;
import com.ruoyi.common.core.domain.model.LoginUser;
import com.ruoyi.common.enums.UserStatus;
import com.ruoyi.common.exception.ServiceException;
import com.ruoyi.common.utils.MessageUtils;
import com.ruoyi.common.utils.StringUtils;
import com.ruoyi.system.service.ISysUserService;

import javax.annotation.Resource;

/**
 * 用户验证处理
 *
 * @author ruoyi
 */
@Service
public class UserDetailsServiceImpl implements UserDetailsService
{
    private static final Logger log = LoggerFactory.getLogger(UserDetailsServiceImpl.class);

    @Autowired
    private ISysUserService userService;

    @Resource
    private IAsUserService asUserService;

    @Autowired
    private SysPasswordService passwordService;

    @Autowired
    private SysPermissionService permissionService;

    @Override
    public UserDetails loadUserByUsername(String username) throws UsernameNotFoundException
    {
        /** 获取到用户类型*/
        Authentication authentication = AuthenticationContextHolder.getContext();
        if (authentication != null && authentication.getDetails() != null) {
            String userType = (String) authentication.getDetails();
            if(Constants.USER_TYPE_PC.equals(userType)){
                // PC用户登录
                SysUser user = userService.selectUserByUserName(username);
                if (StringUtils.isNull(user))
                {
                    log.info("登录用户：{} 不存在.", username);
                    throw new ServiceException(MessageUtils.message("user.not.exists"));
                }
                else if (UserStatus.DELETED.getCode().equals(user.getDelFlag()))
                {
                    log.info("登录用户：{} 已被删除.", username);
                    throw new ServiceException(MessageUtils.message("user.password.delete"));
                }
                else if (UserStatus.DISABLE.getCode().equals(user.getStatus()))
                {
                    log.info("登录用户：{} 已被停用.", username);
                    throw new ServiceException(MessageUtils.message("user.blocked"));
                }

                passwordService.validate(user);

                return createLoginUser(user);
            }else {
                // app用户登录
                AsUser user = asUserService.selectUserByUserName(username);
                if (StringUtils.isNull(user))
                {
                    log.info("登录用户：{} 不存在.", username);
                    throw new ServiceException(MessageUtils.message("user.not.exists"));
                }
                else if (UserStatus.DELETED.getCode().equals(user.getDelFlag()))
                {
                    log.info("登录用户：{} 已被删除.", username);
                    throw new ServiceException(MessageUtils.message("user.password.delete"));
                }
                else if (UserStatus.DISABLE.getCode().equals(user.getStatus()))
                {
                    log.info("登录用户：{} 已被停用.", username);
                    throw new ServiceException(MessageUtils.message("user.blocked"));
                }
                UserDetails loginUser = createLoginUser(user);
                return loginUser;
            }

        } else {
            log.info("用户类型未正常获取");
            throw new ServiceException(MessageUtils.message("user.not.type"));
        }
    }

    public UserDetails createLoginUser(SysUser user)
    {
        return new LoginUser(user.getUserId(), user.getDeptId(), user, permissionService.getMenuPermission(user));
    }

    public UserDetails createLoginUser(AsUser user)
    {
        return new LoginUser(user.getUserId(), user);
    }
}
