package com.ruoyi.system.mapper;

import com.baomidou.mybatisplus.core.mapper.BaseMapper;
import com.ruoyi.system.domain.AsDevice;
import org.apache.ibatis.annotations.Param;

import java.util.List;

/**
 * 设备Mapper接口
 *
 * @author 邱贞招
 * @date 2024-04-05
 */
public interface AsDeviceMapper extends BaseMapper<AsDevice>
{
    /**
     * 查询设备
     *
     * @param deviceId 设备主键
     * @return 设备
     */
    public AsDevice selectAsDeviceByDeviceId(Long deviceId);

    /**
     * 根据mac号查询设备信息
     *
     * @param mac 设备列表主键
     * @return 设备列表
     */
    public AsDevice selectAsDeviceByMac(String mac);

    /**
     * 根据SN查询设备信息
     *
     * @param sn 设备SN
     * @return 设备列表
     */
    public AsDevice selectAsDeviceBySn(String sn);


    /**
     * 查询设备列表
     *
     * @param asDevice 设备
     * @return 设备集合
     */
    public List<AsDevice> selectAsDeviceList(AsDevice asDevice);

    /**
     * 新增设备
     *
     * @param asDevice 设备
     * @return 结果
     */
    public int insertAsDevice(AsDevice asDevice);

    /**
     * 修改设备
     *
     * @param asDevice 设备
     * @return 结果
     */
    public int updateAsDevice(AsDevice asDevice);

    /**
     * 根据SN修改设备
     *
     * @param asDevice 设备
     * @return 结果
     */
    public int updateAsDeviceBySn(AsDevice asDevice);

    /**
     * 删除设备
     *
     * @param deviceId 设备主键
     * @return 结果
     */
    public int deleteAsDeviceByDeviceId(Long deviceId);

    /**
     * 批量删除设备
     *
     * @param deviceIds 需要删除的数据主键集合
     * @return 结果
     */
    public int deleteAsDeviceByDeviceIds(Long[] deviceIds);

    /**
     * 有订单车辆
     */
    String getInOrderDevice(@Param("timeStart") String timeStart, @Param("timeEnd") String timeEnd);

    /**
     * 无订单车辆
     */
    String getNoOrderDevice(@Param("timeStart") String timeStart, @Param("timeEnd") String timeEnd);

    public int batchDisable(String[] sns);

}
