package com.ruoyi.system.mapper;

import java.math.BigDecimal;
import java.util.List;
import com.ruoyi.system.domain.EtOrder;
import org.apache.ibatis.annotations.Param;

/**
 * 订单Mapper接口
 *
 * @author 邱贞招
 * @date 2024-04-08
 */
public interface EtOrderMapper
{
    /**
     * 查询订单
     *
     * @param orderId 订单主键
     * @return 订单
     */
    public EtOrder selectEtOrderByOrderId(Long orderId);

    /**
     * 根据订单号查询订单
     *
     * @param orderNo 订单号
     * @return 订单
     */
    public EtOrder selectEtOrderByOrderNo(String orderNo);

    /**
     * 根据微信内部订单号查询订单
     *
     * @param outTradeNo 订单号
     * @return 订单
     */
    public EtOrder selectEtOrderByOutTradeNo(String outTradeNo);

    /**
     * 查询订单列表
     *
     * @param etOrder 订单
     * @return 订单集合
     */
    public List<EtOrder> selectEtOrderList(EtOrder etOrder);

    /**
     * 新增订单
     *
     * @param etOrder 订单
     * @return 结果
     */
    public int insertEtOrder(EtOrder etOrder);

    /**
     * 修改订单
     *
     * @param etOrder 订单
     * @return 结果
     */
    public int updateEtOrder(EtOrder etOrder);

    /**
     * 根据订单号修改订单
     *
     * @param etOrder 订单
     * @return 结果
     */
    public int updateEtOrderByOrderNo(EtOrder etOrder);

    /**
     * 删除订单
     *
     * @param orderId 订单主键
     * @return 结果
     */
    public int deleteEtOrderByOrderId(Long orderId);

    /**
     * 批量删除订单
     *
     * @param orderIds 需要删除的数据主键集合
     * @return 结果
     */
    public int deleteEtOrderByOrderIds(Long[] orderIds);

    /**
     * 查询当前用户是否有正在进行中的订单
     *
     * @param userId 用户id
     * @param orderNo 排除当前订单号
     * @return 结果
     */
    public List<EtOrder> isInOrder(@Param("userId") Long userId,@Param("orderNo") String orderNo);

    /**
     * 检验预约订单只能有一个
     *
     * @param userId 用户id
     * @return 结果
     */
    int checkIsUnique(Long userId);

    /**
     * 总收入
     */
    String getTotalIncome(@Param("timeStart") String timeStart, @Param("timeEnd") String timeEnd);

    /**
     * 累计待支付
     */
    String getTotalUnpaid(@Param("timeStart") String timeStart, @Param("timeEnd") String timeEnd);

    /**
     * 已支付
     */
    String getTotalPaid(@Param("timeStart") String timeStart, @Param("timeEnd") String timeEnd);

    /**
     * 已退款
     */
    String getTotalRefund(@Param("timeStart") String timeStart, @Param("timeEnd") String timeEnd);

    /**
     * 骑行已支付
     */
    String getTotalRidingFee(@Param("timeStart") String timeStart, @Param("timeEnd") String timeEnd);

    /**
     * 骑行已退款
     */
    String getTotalRidingRefund(@Param("timeStart") String timeStart, @Param("timeEnd") String timeEnd);

    /**
     * 调度费已支付
     */
    String getTotalDispatchFee(@Param("timeStart") String timeStart, @Param("timeEnd") String timeEnd);

    /**
     * 调度费已退款
     */
    String getTotalDispatchRefund(@Param("timeStart") String timeStart, @Param("timeEnd") String timeEnd);

    /**
     * 预约费已支付
     */
    String getTotalAppointmentFee(@Param("timeStart") String timeStart, @Param("timeEnd") String timeEnd);

    /**
     * 预约费已退款
     */
    String getTotalAppointmentRefund(@Param("timeStart") String timeStart, @Param("timeEnd") String timeEnd);

    /**
     * 管理费已支付
     */
    String getTotalManageFee(@Param("timeStart") String timeStart, @Param("timeEnd") String timeEnd);

    /**
     * 管理费已退款
     */
    String getTotalManageRefund(@Param("timeStart") String timeStart, @Param("timeEnd") String timeEnd);


    /**
     * 已支付订单
     */
    String getPaidOrder(@Param("timeStart") String timeStart, @Param("timeEnd") String timeEnd);

    /**
     * 骑行中订单
     */
    String getRidingOrder(@Param("timeStart") String timeStart, @Param("timeEnd") String timeEnd);

    /**
     * 退款订单
     */
    String getRefundOrder(@Param("timeStart") String timeStart, @Param("timeEnd") String timeEnd);

    /**
     * 待支付订单
     */
    String getUnpaidOrder(@Param("timeStart") String timeStart, @Param("timeEnd") String timeEnd);

    /**
     * 手续费
     */
    BigDecimal getServiceFee(@Param("startDateStr") String startDateStr , @Param("endDateStr") String endDateStr);

    /**
     * 支付金额
     */
    BigDecimal getPayFee(@Param("startDateStr") String startDateStr , @Param("endDateStr") String endDateStr, @Param("sn") String sn);

    /**
     * 已退款
     */
    BigDecimal getRefundFee(@Param("startDateStr") String startDateStr , @Param("endDateStr") String endDateStr, @Param("sn") String sn);

    /**
     * 收入
     */
    BigDecimal getIncome(@Param("startDateStr") String startDateStr , @Param("endDateStr") String endDateStr, @Param("sn") String sn);

    /**
     * 最近一笔订单
     */
    EtOrder selectLatestOrder(EtOrder etOrder);
}
