package com.ruoyi.system.service;

import com.ruoyi.common.core.domain.entity.AsUser;
import com.ruoyi.system.domain.vo.AuthenticationVo;

import java.util.List;

/**
 * 用户 业务层
 *
 * @author ruoyi
 */
public interface IAsUserService
{
    /**
     * 根据条件分页查询用户列表
     *
     * @param user 用户信息
     * @return 用户信息集合信息
     */
    public List<AsUser> selectUserList(AsUser user);

    /**
     * 根据条件分页查询已分配用户角色列表
     *
     * @param user 用户信息
     * @return 用户信息集合信息
     */
    public List<AsUser> selectAllocatedList(AsUser user);

    /**
     * 根据条件分页查询未分配用户角色列表
     *
     * @param user 用户信息
     * @return 用户信息集合信息
     */
    public List<AsUser> selectUnallocatedList(AsUser user);

    /**
     * 通过用户名查询用户
     *
     * @param userName 用户名
     * @return 用户对象信息
     */
    public AsUser selectUserByUserName(String userName);

    /**
     * 通过用户ID查询用户
     *
     * @param userId 用户ID
     * @return 用户对象信息
     */
    public AsUser selectUserById(Long userId);



    /**
     * 校验用户名称是否唯一
     *
     * @param user 用户信息
     * @return 结果
     */
    public boolean checkUserNameUnique(AsUser user);

    /**
     * 校验手机号码是否唯一
     *
     * @param user 用户信息
     * @return 结果
     */
    public boolean checkPhoneUnique(AsUser user);

    /**
     * 校验email是否唯一
     *
     * @param user 用户信息
     * @return 结果
     */
    public boolean checkEmailUnique(AsUser user);

    /**
     * 新增用户信息
     *
     * @param user 用户信息
     * @return 结果
     */
    public int insertUser(AsUser user);

    /**
     * 注册用户信息
     *
     * @param user 用户信息
     * @return 结果
     */
    public boolean registerUser(AsUser user);

    /**
     * 修改用户信息
     *
     * @param user 用户信息
     * @return 结果
     */
    public int updateUser(AsUser user);

    /**
     * 修改用户状态
     *
     * @param user 用户信息
     * @return 结果
     */
    public int updateUserStatus(AsUser user);

    /**
     * 修改用户基本信息
     *
     * @param user 用户信息
     * @return 结果
     */
    public int updateUserProfile(AsUser user);

    /**
     * 修改用户头像
     *
     * @param userName 用户名
     * @param avatar 头像地址
     * @return 结果
     */
    public boolean updateUserAvatar(String userName, String avatar);

    /**
     * 重置用户密码
     *
     * @param user 用户信息
     * @return 结果
     */
    public int resetPwd(AsUser user);

    /**
     * 重置用户密码
     *
     * @param userName 用户名
     * @param password 密码
     * @return 结果
     */
    public int resetUserPwd(String userName, String password);

    /**
     * 通过用户ID删除用户
     *
     * @param userId 用户ID
     * @return 结果
     */
    public int deleteUserById(Long userId);

    /**
     * 批量删除用户信息
     *
     * @param userIds 需要删除的用户ID
     * @return 结果
     */
    public int deleteUserByIds(Long[] userIds);

    /**
     * 导入用户数据
     *
     * @param userList 用户数据列表
     * @param isUpdateSupport 是否更新支持，如果已存在，则进行更新数据
     * @param operName 操作用户
     * @return 结果
     */
    public String importUser(List<AsUser> userList, Boolean isUpdateSupport, String operName);

    /**
     * 通过用户手机号查询用户
     *
     * @param phone 手机号
     * @return 用户对象信息
     */
    AsUser selectUserByPhone(String phone);

    /**
     * 通过openid查询用户
     *
     * @param openid
     * @return 用户对象信息
     */
    AsUser selectUserByWxopenid(String openid);

    /**
     * 检测用户是否有充值过押金
     * @param userId
     */
    public Boolean checkIsDeposit(Long userId);

    /**
     * 新增用户
     */
    String getNewUser(String powerStart, String powerEnd);

    /**
     * 总用户
     */
    String getTotalUser(String powerStart, String powerEnd);

    /**
     * 租赁用户
     */
    String getLeaseUser(String powerStart, String powerEnd);

    /**
     * 绑定系统用户
     */
    int bandSystemUser(AsUser user);

    /**
     * 实名认证
     *
     */
    Object authentication(AuthenticationVo authenticationVo);
}
