package com.ruoyi.system.service;

import java.util.List;

import com.baomidou.mybatisplus.extension.service.IService;
import com.ruoyi.common.core.domain.entity.AsUser;
import com.ruoyi.system.domain.EtOperatingArea;
import com.ruoyi.system.domain.SysStudent;

/**
 * 运营区Service接口
 *
 * @author 邱贞招
 * @date 2024-04-06
 */
public interface IEtOperatingAreaService extends IService<EtOperatingArea>
{
    /**
     * 查询运营区
     *
     * @param areaId 运营区主键
     * @return 运营区
     */
    public EtOperatingArea selectEtOperatingAreaByAreaId(Long areaId);

    /**
     * 查询运营区列表
     *
     * @param etOperatingArea 运营区
     * @return 运营区集合
     */
    public List<EtOperatingArea> selectEtOperatingAreaList(EtOperatingArea etOperatingArea);

    /**
     * 新增运营区
     *
     * @param etOperatingArea 运营区
     * @return 结果
     */
    public int insertEtOperatingArea(EtOperatingArea etOperatingArea);

    /**
     * 修改运营区
     *
     * @param etOperatingArea 运营区
     * @return 结果
     */
    public int updateEtOperatingArea(EtOperatingArea etOperatingArea);

    /**
     * 批量删除运营区
     *
     * @param areaIds 需要删除的运营区主键集合
     * @return 结果
     */
    public int deleteEtOperatingAreaByAreaIds(Long[] areaIds);

    /**
     * 删除运营区信息
     *
     * @param areaId 运营区主键
     * @return 结果
     */
    public int deleteEtOperatingAreaByAreaId(Long areaId);

    /**
     * 获取分区选择框列表
     *
     * @return 字典类型集合信息
     */
    public List<EtOperatingArea> selectAreaAll();


    /**
     * 校验运营区是否唯一
     *
     * @param etOperatingArea 运营区信息
     * @return 结果
     */
    public boolean checkAreaNameUnique(EtOperatingArea etOperatingArea);

    /**
     * 根据定位获取运营区详细信息
     *
     * @param longitude 经度
     * @param latitude 纬度
     * @return 结果
     */
    EtOperatingArea getAreaInfoByLocation(String longitude, String latitude);

    /**
     * 根据部门ID获取运营区选择框列表
     *
     * @param deptId 用户ID
     * @return 选中运营区ID列表
     */
    public List<Long> selectAreaListByDeptId(Long deptId);
}
