package com.ruoyi.system.service.impl;

import cn.hutool.core.util.ObjectUtil;
import cn.hutool.core.util.StrUtil;
import com.alibaba.fastjson2.JSON;
import com.alibaba.fastjson2.JSONObject;
import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.baomidou.mybatisplus.extension.service.impl.ServiceImpl;
import com.ruoyi.common.constant.HttpStatus;
import com.ruoyi.common.constant.IotConstants;
import com.ruoyi.common.constant.ServiceConstants;
import com.ruoyi.common.exception.ServiceException;
import com.ruoyi.common.utils.CommonUtil;
import com.ruoyi.common.utils.DateUtils;
import com.ruoyi.common.utils.http.HttpUtils;
import com.ruoyi.common.utils.map.GeoUtils;
import com.ruoyi.common.utils.onenet.IotUtil;
import com.ruoyi.common.utils.onenet.Token;
import com.ruoyi.common.utils.uuid.IdUtils;
import com.ruoyi.system.domain.*;
import com.ruoyi.system.domain.response.OrderResponse;
import com.ruoyi.system.domain.vo.DeviceNumVo;
import com.ruoyi.system.domain.vo.EtOrderVo;
import com.ruoyi.system.domain.vo.IntervalRuleVo;
import com.ruoyi.system.domain.vo.StartingRuleVo;
import com.ruoyi.system.mapper.AsDeviceMapper;
import com.ruoyi.system.service.*;
import lombok.SneakyThrows;
import lombok.extern.slf4j.Slf4j;
import org.apache.commons.lang3.ObjectUtils;
import org.jetbrains.annotations.NotNull;
import org.locationtech.jts.geom.Geometry;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.beans.factory.annotation.Value;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;
import org.springframework.transaction.support.TransactionTemplate;

import javax.annotation.Resource;
import java.math.BigDecimal;
import java.math.RoundingMode;
import java.util.ArrayList;
import java.util.Date;
import java.util.List;
import java.util.concurrent.TimeUnit;

/**
 * 设备Service业务层处理
 *
 * @author 邱贞招
 * @date 2024-04-05
 */
@Slf4j
@Service
public class AsDeviceServiceImpl extends ServiceImpl<AsDeviceMapper, AsDevice> implements IAsDeviceService
{
    @Resource
    private AsDeviceMapper asDeviceMapper;

    @Autowired
    private TransactionTemplate transactionTemplate;

    @Resource
    private IEtOperatingAreaService etOperatingAreaService;

    @Autowired
    private IEtOrderService etOrderService;

    @Autowired
    private ISysConfigService sysConfigService;

    @Autowired
    private IEtAdminOrderService etAdminOrderService;

    @Autowired
    private IEtFeeRuleService etFeeRuleService;

    @Autowired
    private IEtParkingAreaService parkingAreaService;

    @Resource
    private ISysDictDataService sysDictDataService;

    @Resource
    private IEtTripLogService tripLogService;

    @Resource
    private IEtModelService etModelService;



    @Value(value = "${iot.iotUrl}")
    private String iotUrl;

    @Value(value = "${iot.productId}")
    private String productId;

    @Value(value = "${iot.timeout}")
    private String timeout;

    /**
     * 查询设备
     *
     * @param deviceId 设备主键
     * @return 设备
     */
    @Override
    public AsDevice selectAsDeviceByDeviceId(Long deviceId)
    {
        return asDeviceMapper.selectAsDeviceByDeviceId(deviceId);
    }

    /**
     * 根据mac查询车辆实时信息
     *
     * @param mac 设备mac
     * @return 设备
     */
    @Override
    public AsDevice selectAsDeviceByMac(String mac) {
        return asDeviceMapper.selectAsDeviceByMac(mac);
    }

    /**
     * 根据sn号查询车辆实时信息
     *
     * @param sn 设备sn
     * @return 设备
     */
    @SneakyThrows
    @Override
    public AsDevice selectAsDeviceBySn(String sn) {
        AsDevice asDevice = asDeviceMapper.selectAsDeviceBySn(sn);
        if (ObjectUtil.isNull(asDevice)) {
            throw new ServiceException("设备不存在");
        }
        Long areaId = asDevice.getAreaId();
        EtOperatingArea etOperatingArea;
        if (ObjectUtil.isNotNull(areaId)) {
            etOperatingArea = etOperatingAreaService.selectEtOperatingAreaByAreaId(areaId);
        }else{
            throw new ServiceException("区域信息不存在");
        }
        if(ObjectUtil.isNotNull(etOperatingArea)){
            asDevice.setAreaName(etOperatingArea.getAreaName());
        }
        EtModel model = etModelService.selectEtModelByModelId(asDevice.getModelId());
        if(ObjectUtil.isNotNull(model)){
            asDevice.setModel(model.getModel());
        }
        asDevice.setLocation(CommonUtil.getAddressByGeo(asDevice.getLongitude() + ","+asDevice.getLatitude()));
        //正在进行中的订单
        EtOrder order = new EtOrder();
        order.setType(ServiceConstants.ORDER_TYPE_RIDING);
        order.setStatus(ServiceConstants.ORDER_STATUS_RIDING);
        order.setSn(asDevice.getSn());
        List<EtOrder> etOrders = etOrderService.selectEtOrderList(order);
        asDevice.setEtOrders(etOrders);
        return asDevice;
    }

    /**
     * 查询设备列表
     *
     * @param asDevice 设备
     * @return 设备
     */
    @Override
    public List<AsDevice> selectAsDeviceList(AsDevice asDevice)
    {
        List<AsDevice> asDevices = asDeviceMapper.selectAsDeviceList(asDevice);
        for (AsDevice asDevice1:asDevices){
            Long areaId = asDevice1.getAreaId();
            if (ObjectUtil.isNotNull(areaId)){
                EtOperatingArea etOperatingArea = etOperatingAreaService.selectEtOperatingAreaByAreaId(areaId);
                asDevice1.setAreaName(etOperatingArea.getAreaName());
            }
            Long modelId = asDevice1.getModelId();
            if (ObjectUtil.isNotNull(modelId)){
                EtModel model  = etModelService.selectEtModelByModelId(modelId);
                if(ObjectUtil.isNotNull(model)){
                    asDevice1.setModel(model.getModel());
                }
            }
            String status = asDevice1.getStatus();
            if(ObjectUtil.isNotNull(status)){
                String typeName = sysDictDataService.selectDictLabel("as_device_status", status);
                asDevice1.setStatusStr(typeName);
            }
        }
        return asDevices;
    }

    /**
     * 新增设备
     *
     * @param asDevice 设备
     * @return 结果
     */
    @Override
    public int insertAsDevice(AsDevice asDevice)
    {
        asDevice.setCreateTime(DateUtils.getNowDate());
        return asDeviceMapper.insertAsDevice(asDevice);
    }

    /**
     * 修改设备
     *
     * @param asDevice 设备
     * @return 结果
     */
    @Override
    public int updateAsDevice(AsDevice asDevice)
    {
        asDevice.setUpdateTime(DateUtils.getNowDate());
        int i = asDeviceMapper.updateAsDevice(asDevice);
        //预约后开始计费

        return i;
    }

    /**
     * 根据SN修改设备
     *
     * @param asDevice 设备
     * @return 结果
     */
    @Override
    public int updateAsDeviceBySn(AsDevice asDevice)
    {
        asDevice.setUpdateTime(DateUtils.getNowDate());
        return asDeviceMapper.updateAsDeviceBySn(asDevice);
    }

    /**
     * 批量删除设备
     *
     * @param deviceIds 需要删除的设备主键
     * @return 结果
     */
    @Override
    public int deleteAsDeviceByDeviceIds(Long[] deviceIds)
    {
        return asDeviceMapper.deleteAsDeviceByDeviceIds(deviceIds);
    }

    /**
     * 删除设备信息
     *
     * @param deviceId 设备主键
     * @return 结果
     */
    @Override
    public int deleteAsDeviceByDeviceId(Long deviceId)
    {
        return asDeviceMapper.deleteAsDeviceByDeviceId(deviceId);
    }

    /**
     * 根据定位获取运营区信息，并返回所有车辆定位
     *   a. 将前端的边界值获取后转成geometry格式
     *   b. 模拟gps定位根据定位计算在哪个运营区内
     *   c. 获取到运营区信息，查询出该运营区下所有的车辆信息并获取所有车辆的定位返回前端
     * @param longitude 经度
     * @param latitude 纬度
     * @return 结果
     */
    @Override
    public List<AsDevice> vehicleLocalization(String longitude, String latitude) {
        List<EtOperatingArea> etOperatingAreas = etOperatingAreaService.selectEtOperatingAreaList(new EtOperatingArea());
        EtOperatingArea area = null;
        for(EtOperatingArea etOperatingArea:etOperatingAreas){
            Geometry geometry = GeoUtils.fromWkt(etOperatingArea.getBoundary());
            Boolean inCircle = GeoUtils.isInCircle(longitude, latitude, geometry);
            if(inCircle){
                area = etOperatingArea;
                break;
            }
        }
        List<AsDevice> asDevices = new ArrayList<>();
        if(ObjectUtil.isNotNull(area)){
            /** c. 获取到运营区信息，查询出该运营区下所有的车辆信息并获取所有车辆的定位返回前端*/
            log.info("在【{}】运营区内",area.getAreaName());
            QueryWrapper<AsDevice> wrapper = new QueryWrapper<>();
            wrapper.eq("area_id", area.getAreaId());
            wrapper.in("status","1");//0 未上架，1-正常
            asDevices = asDeviceMapper.selectList(wrapper);
            return asDevices;
        }else{
            log.info("不在任何运营区内");
            return asDevices;
        }
    }

    /**
     * 所有车辆定位
     */
    @Override
    public List<AsDevice> allVehicleInfo(String powerStart, String powerEnd, String status,String sort) {
        QueryWrapper<AsDevice> wrapper = new QueryWrapper<>();
        if(StrUtil.isNotBlank(powerStart) && StrUtil.isNotBlank(powerEnd)){
            wrapper.between("remaining_power", Integer.parseInt(powerStart), Integer.parseInt(powerEnd));
        }
        if(StrUtil.isNotBlank(status)){
            wrapper.in("status",status);
        }
        if(StrUtil.isNotBlank(sort)){
            wrapper.orderBy(true,!StrUtil.equals(sort, "desc"),"remaining_power");
        }
        List<AsDevice> asDevices = asDeviceMapper.selectList(wrapper);
        for (AsDevice asDevice:asDevices){
            Long modelId = asDevice.getModelId();
            if (ObjectUtil.isNotNull(modelId)){
                EtModel model = etModelService.selectEtModelByModelId(modelId);
                if(ObjectUtil.isNotNull(model)){
                    asDevice.setModel(model.getModel());
                }
            }
        }
        return asDevices;
    }

    /**
     * 查询车辆数量
     */
    @Override
    public DeviceNumVo allVehicleNum(String powerStart, String powerEnd) {
        DeviceNumVo deviceNumVo = new DeviceNumVo();
        QueryWrapper<AsDevice> wrapper = new QueryWrapper<>();
        if(StrUtil.isNotBlank(powerStart) && StrUtil.isNotBlank(powerEnd)){
            wrapper.between("remaining_power", powerStart,powerEnd);
        }
        Integer allNum = asDeviceMapper.selectCount(wrapper);
        deviceNumVo.setAllNum(allNum);//所有车辆

        Integer ridingNum = setNum(powerStart, powerEnd,ServiceConstants.VEHICLE_STATUS_IN_USING);
        deviceNumVo.setRidingNum(ridingNum);//骑行中

        Integer temporarilyLockNum = setNum(powerStart, powerEnd,ServiceConstants.VEHICLE_STATUS_TEMPORARILY_LOCK);
        deviceNumVo.setTemporarilyLockNum(temporarilyLockNum);//临时锁车

        Integer disabledNum = setNum(powerStart, powerEnd,ServiceConstants.VEHICLE_STATUS_IN_OFFLINE);
        deviceNumVo.setDisabledNum(disabledNum);//已禁用

        Integer normalNum = setNum(powerStart, powerEnd,ServiceConstants.VEHICLE_STATUS_NORMAL);
        deviceNumVo.setNormalNum(normalNum);//正常待租

        QueryWrapper<AsDevice> wrapperForOffline = new QueryWrapper<>();
        wrapperForOffline.eq("online_status","0");//在线状态：0-不在线；1-在线
        if(StrUtil.isNotBlank(powerStart) && StrUtil.isNotBlank(powerEnd)){
            wrapperForOffline.between("remaining_power", powerStart,powerEnd);
        }
        Integer offlineNum = asDeviceMapper.selectCount(wrapperForOffline);
        deviceNumVo.setOfflineNum(offlineNum);//已离线

        Integer inAppointmentNum = setNum(powerStart, powerEnd, ServiceConstants.VEHICLE_STATUS_IN_APPOINTMENT);
        deviceNumVo.setInAppointmentNum(inAppointmentNum);//预约中

        QueryWrapper<AsDevice> wrapperForDispatch = new QueryWrapper<>();
        wrapperForDispatch.or(i -> i.eq("in_parking_area", 1).or().eq("in_operating_area", 1));
        if(StrUtil.isNotBlank(powerStart) && StrUtil.isNotBlank(powerEnd)){
            wrapperForDispatch.between("remaining_power", powerStart,powerEnd);
        }
        Integer dispatchNum = asDeviceMapper.selectCount(wrapperForDispatch);
        deviceNumVo.setDispatchNum(dispatchNum);//调度设备
        return deviceNumVo;
    }

    private Integer setNum(String powerStart, String powerEnd,String status) {
        QueryWrapper<AsDevice> wrapper = new QueryWrapper<>();
        wrapper.eq("status",status);
        if(StrUtil.isNotBlank(powerStart) && StrUtil.isNotBlank(powerEnd)){
            wrapper.between("remaining_power", powerStart,powerEnd);
        }
        return asDeviceMapper.selectCount(wrapper);

    }


    /**
     * 编号开锁
     * @param orderVo 订单对象
     *      1.远程开锁
     *      2.更新车辆状态
     *      3.创建订单
     * @return
     */
    @SneakyThrows
    @Override
    @Transactional
    public OrderResponse snSwitch(EtOrderVo orderVo) {
        OrderResponse response = new OrderResponse();
        String orderNo = "";
        if(ObjectUtil.isNull(orderVo.getOrderNo())){
            orderNo = IdUtils.randomUUID2();
        }else{
            orderNo = orderVo.getOrderNo();
        }
        String sessionId = IdUtils.randomUUIDByDigit(8);
        AsDevice asDevice = asDeviceMapper.selectAsDeviceBySn(orderVo.getSn());
        /** 1.获取token*/
        String token = Token.getToken();
        String finalOrderNo = orderNo;
        Boolean execute = transactionTemplate.execute(e -> {
            /** 2.发送命令*/
            sendCommand(asDevice.getMac(), token,IotConstants.COMMAND_OPEN,"编号开锁");
            //间隔1秒
//            try {
//                Thread.sleep(1000);
//            } catch (InterruptedException ex) {
//                ex.printStackTrace();
//            }
//            sendCommand(asDevice.getMac(), token,IotConstants.COMMAND_PLAY0,"编号开锁播报");
            /** 3.更新车辆状态*/
            asDevice.setLockStatus(ServiceConstants.LOCK_STATUS_OPEN);
            asDevice.setStatus(ServiceConstants.VEHICLE_STATUS_IN_USING);
            int device = asDeviceMapper.updateAsDevice(asDevice);
            if(device==0){
                log.info("【扫码/编号开锁骑行】更新车辆状态失败");
                return Boolean.FALSE;
            }
            /** 4.如果有预约订单则更新订单的预约结束时间，如果没有预约，则创建订单*/
            EtOrder order = etOrderService.selectEtOrderByOrderNo(finalOrderNo);
            if(ObjectUtil.isNotNull(order)){//有订单号：可能是套餐也可能是有预约
                if(ObjectUtil.isNotNull(order.getRuleId())){//套餐
                    log.info("【扫码/编号开锁骑行】---预约扫码骑行");
                    order.setSn(asDevice.getSn());
                }else{
                    throw new ServiceException("ruleId未传");
                }
                order.setStatus(ServiceConstants.ORDER_STATUS_RIDING);
                order.setUnlockTime(DateUtils.getNowDate());
                order.setAppointmentEndTime(DateUtils.getNowDate());
                //计算预约费
                calculateAppointmentFee(order);
                int update = etOrderService.updateEtOrder(order);
                if(update==0){
                    log.info("【扫码/编号开锁骑行】更新订单失败");
                }
            }else{
                log.info("【扫码/编号开锁骑行】---无订单号,正常开锁骑行订单");
                order = etOrderService.createOrder(orderVo, finalOrderNo);
                int etOrder = etOrderService.insertEtOrder(order);
                if(etOrder==0){
                    log.info("【扫码/编号开锁骑行】保存订单失败");
                    return Boolean.FALSE;
                }
            }
            /** 5.记录行程*/
            int tripLog = tripLogService.tripLog(order.getOrderNo(),order.getSn(),ServiceConstants.TRIP_LOG_TYPE_UNLOCK_RIDE);
            if(tripLog==0){
                log.info("【扫码/编号开锁骑行】记录行程失败");
                throw new ServiceException("【扫码/编号开锁骑行】记录行程失败");
            }
            log.info("【扫码/编号开锁骑行】车辆开锁成功");
            return Boolean.TRUE;
        });
        if(!execute)throw new ServiceException("【扫码/编号开锁骑行】开锁失败");
        response.setOrderNo(orderNo);
        response.setSessionId(sessionId);
        return response;
    }

    /**
     * 管理员开锁
     */
    @SneakyThrows
    @Override
    public Boolean unlocking(String sn) {
        AsDevice asDevice = asDeviceMapper.selectAsDeviceBySn(sn);
        /** 1.获取token*/
        String token = Token.getToken();
        Boolean execute = transactionTemplate.execute(e -> {
            /** 2.发送命令*/
            sendCommand(asDevice.getMac(), token,IotConstants.COMMAND_OPEN,"管理员开锁");
            //间隔1秒
//            try {
//                Thread.sleep(500);
//            } catch (InterruptedException ex) {
//                ex.printStackTrace();
//            }
//            sendCommand(asDevice.getMac(), token,IotConstants.COMMAND_PLAY0,"管理员开锁播报");
            return Boolean.TRUE;
        });
        if(!execute)throw new ServiceException("管理员开锁失败");
        return Boolean.TRUE;
    }

    /**
     * 车辆上线
     */
    @SneakyThrows
    @Override
    public Boolean online(String sn) {
        AsDevice asDevice = asDeviceMapper.selectAsDeviceBySn(sn);
        asDevice.setStatus(ServiceConstants.VEHICLE_STATUS_NORMAL_STR);
        int device = asDeviceMapper.updateAsDevice(asDevice);
        if(device==0){
            log.info("车辆上线状态失败");
            return Boolean.FALSE;
        }
        return Boolean.TRUE;
    }

    /**
     * 车辆下线
     */
    @SneakyThrows
    @Override
    public Boolean offline(String sn) {
        AsDevice asDevice = asDeviceMapper.selectAsDeviceBySn(sn);
        asDevice.setStatus(ServiceConstants.VEHICLE_STATUS_IN_OFFLINE);
        int device = asDeviceMapper.updateAsDevice(asDevice);
        if(device==0){
            log.info("车辆下线状态失败");
            return Boolean.FALSE;
        }
        return Boolean.TRUE;
    }

    /**
     * 根据条件模糊查询车辆信息
     * type:1-sn,2-mac,3-车牌号
     */
    @Override
    public List<AsDevice> deviceSearch(String key, String type) {
        switch (type){
            case "1":
                QueryWrapper<AsDevice> sn = new QueryWrapper<AsDevice>().like("sn", key);
                return asDeviceMapper.selectList(sn);
            case "2":
                QueryWrapper<AsDevice> mac = new QueryWrapper<AsDevice>().like("mac", key);
                return asDeviceMapper.selectList(mac);
            case "3":
                QueryWrapper<AsDevice> vehicleNum = new QueryWrapper<AsDevice>().like("vehicle_num", key);
                return asDeviceMapper.selectList(vehicleNum);
            default:
                return null;
        }
    }

    /**
     * 运营中车辆
     */
    @Override
    public String getInOperationDevice(String timeStart, String timeEnd) {
        if(ObjectUtil.isNotEmpty(timeStart)&&ObjectUtil.isNotEmpty(timeEnd)){
            QueryWrapper<AsDevice> wrapper = new QueryWrapper<AsDevice>().eq("status", ServiceConstants.VEHICLE_STATUS_IN_USING);
            return asDeviceMapper.selectCount(wrapper)+"";
        }
        return null;
    }

    /**
     * 有订单车辆
     */
    @Override
    public String getInOrderDevice(String timeStart, String timeEnd) {
        if(ObjectUtil.isNotEmpty(timeStart)&&ObjectUtil.isNotEmpty(timeEnd)){
            return asDeviceMapper.getInOrderDevice(timeStart,timeEnd);
        }
        return null;
    }

    /**
     * 无订单车辆
     */
    @Override
    public String getNoOrderDevice(String timeStart, String timeEnd) {
        if(ObjectUtil.isNotEmpty(timeStart)&&ObjectUtil.isNotEmpty(timeEnd)){
            return asDeviceMapper.getNoOrderDevice(timeStart,timeEnd);
        }
        return null;
    }

    /**
     * 根据车辆型号查询车辆数量
     */
    @Override
    public Integer selectCountByModelId(Long modelId) {
        QueryWrapper<AsDevice> wrapper = new QueryWrapper<>();
        wrapper.eq("model_id", modelId);
        Integer integer = asDeviceMapper.selectCount(wrapper);
        return integer;
    }

    /**
     * 根据运营区查询车辆数量
     */
    @Override
    public Integer selectCountByAreaId(Long areaId) {
        QueryWrapper<AsDevice> wrapper = new QueryWrapper<>();
        wrapper.eq("area_id", areaId);
        Integer integer = asDeviceMapper.selectCount(wrapper);
        return integer;
    }

    @Transactional
    @Override
    /** 发送命令*/
    public void sendCommand(String mac, String token,String command,String type) {
        String param = "device_name=" + mac + "&product_id=" + productId +"&timeout=" + timeout;
        String sendUrl = iotUrl+ IotConstants.ADDS_COMMAND + "?"+param;
        String result = HttpUtils.sendPostWithToken(sendUrl, command, token);
        log.info("【"+type+"】===>IOT请求调用结果:【{}】",result);
        JSONObject paramsObj = JSON.parseObject(result);
        String code = paramsObj.getString("code");
        //记录命令

        if (!HttpStatus.IOT_SUCCESS.equals(code))
        {
            throw new ServiceException(code+"-----"+ IotUtil.formatMsg(code));
        }
    }

    /**
     * 响铃寻车
     * @param sn
     * @return
     */
    @SneakyThrows
    @Override
    public Boolean ring(String sn) {
        AsDevice asDevice = asDeviceMapper.selectAsDeviceBySn(sn);
        /** TODO  响铃寻车*/
        /** 1.获取token*/
        String token = Token.getToken();
        Boolean execute = transactionTemplate.execute(e -> {
            /** 2.发送命令*/
            sendCommand(asDevice.getMac(), token,IotConstants.COMMAND_PLAY1,"响铃寻车");
            return Boolean.TRUE;
        });
        if(!execute)throw new ServiceException("响铃寻车失败");
        return Boolean.TRUE;
    }

    /**
     * 临时锁车
     * @param orderNo
     * @return
     */
    @Transactional
    @SneakyThrows
    @Override
    public Boolean lock(String sn,String orderNo) {
        EtOrder order = etOrderService.selectEtOrderByOrderNo(orderNo);
        if(order==null)throw new ServiceException("订单不存在");
        if(StrUtil.isBlank(sn)) {
            sn = order.getSn();
        }
        AsDevice asDevice = asDeviceMapper.selectAsDeviceBySn(sn);
        /** 1.获取token*/
        String token = Token.getToken();
        String finalSn = sn;
        Boolean execute = transactionTemplate.execute(e -> {
            /** 2.发送命令*/
            sendCommand(asDevice.getMac(), token,IotConstants.COMMAND_LLOSE,"临时锁车");
            //间隔1秒
//            try {
//                Thread.sleep(500);
//            } catch (InterruptedException ex) {
//                ex.printStackTrace();
//            }
//            sendCommand(asDevice.getMac(), token,IotConstants.COMMAND_PLAY7,"临时锁车播报");
            if(StrUtil.isNotBlank(orderNo)){//有订单号，则是用户临时锁车
                /**  改变车辆状态：4-临时锁车 */
                asDevice.setStatus(ServiceConstants.VEHICLE_STATUS_TEMPORARILY_LOCK);//临时锁车
                asDevice.setLockStatus(ServiceConstants.LOCK_STATUS_CLOSE);
                int device = asDeviceMapper.updateAsDevice(asDevice);
                if(device==0){
                    log.info("【临时解锁】改变车辆状态失败");
                    throw new ServiceException("【临时锁车】改变车辆状态失败");
                }
                /** 5.记录行程*/
                int tripLog = tripLogService.tripLog(orderNo, finalSn,ServiceConstants.TRIP_LOG_TYPE_TEMPORARILY_LOCK);
                if(tripLog==0){
                    log.info("【临时锁车】记录行程失败");
                    throw new ServiceException("【临时锁车】记录行程失败");
                }
            }else{
                /**  改变车辆锁状态：1-开   TODO 是否要改变车辆状态?*/
                asDevice.setLockStatus(ServiceConstants.LOCK_STATUS_CLOSE);
                int device = asDeviceMapper.updateAsDevice(asDevice);
                if(device==0){
                    log.info("【临时解锁】改变车辆状态失败");
                    throw new ServiceException("【临时解锁】改变车辆状态失败");
                }
            }
            return Boolean.TRUE;
        });
        if(!execute)throw new ServiceException("临时锁车失败");
        return Boolean.TRUE;
    }

    /**
     * 临时解锁
     */
    @SneakyThrows
    @Override
    public Boolean unlock(String sn, String orderNo) {
        EtOrder order = etOrderService.selectEtOrderByOrderNo(orderNo);
        if(order==null)throw new ServiceException("订单不存在");
        if(StrUtil.isBlank(sn)) {
            sn = order.getSn();
        }
        AsDevice asDevice = asDeviceMapper.selectAsDeviceBySn(sn);
        /** 1.获取token*/
        String token = Token.getToken();
        String finalSn = sn;
        Boolean execute = transactionTemplate.execute(e -> {
            /** TODO  临时解锁*/
            /** 2.发送命令*/
            sendCommand(asDevice.getMac(), token,IotConstants.COMMAND_OPEN,"临时解锁");
            //间隔1秒
//            try {
//                Thread.sleep(500);
//            } catch (InterruptedException ex) {
//                ex.printStackTrace();
//            }
//            sendCommand(asDevice.getMac(), token,IotConstants.COMMAND_PLAY0,"临时解锁播报");
            if(StrUtil.isNotBlank(orderNo)){//有订单号，则是用户骑行中解锁
                /**  改变车辆状态：3-骑行中 */
                asDevice.setStatus(ServiceConstants.VEHICLE_STATUS_IN_USING);//骑行中
                asDevice.setLockStatus(ServiceConstants.LOCK_STATUS_OPEN);
                int device = asDeviceMapper.updateAsDevice(asDevice);
                if(device==0){
                    log.info("【临时解锁】改变车辆状态失败");
                    throw new ServiceException("【临时解锁】改变车辆状态失败");
                }
                /** 5.记录行程*/
                int tripLog = tripLogService.tripLog(orderNo, finalSn,ServiceConstants.TRIP_LOG_TYPE_TEMPORARILY_UNLOCK);
                if(tripLog==0){
                    log.info("【临时解锁】记录行程失败");
                    throw new ServiceException("【临时解锁】记录行程失败");
                }
            }else{
                /**  改变车辆锁状态：1-开   TODO 是否要改变车辆状态?*/
                asDevice.setLockStatus(ServiceConstants.LOCK_STATUS_OPEN);
                int device = asDeviceMapper.updateAsDevice(asDevice);
                if(device==0){
                    log.info("【临时解锁】改变车辆状态失败");
                    throw new ServiceException("【临时解锁】改变车辆状态失败");
                }
            }
            return Boolean.TRUE;
        });
        if(!execute)throw new ServiceException("临时解锁失败");
        return Boolean.TRUE;
    }

    /**
     * 车辆预约
     */
    @Override
    public OrderResponse deviceAppointment(EtOrderVo appointmentVo) {
        OrderResponse orderResponse = new OrderResponse();
        String orderNo = IdUtils.randomUUID2();
        AsDevice asDevice = asDeviceMapper.selectAsDeviceBySn(appointmentVo.getSn());
        //如果是正常计费模式，则按正常的订单走
        EtOrder order = etOrderService.createOrder(appointmentVo, orderNo);
        int i = etOrderService.insertEtOrder(order);
        if(i==0){
            throw new ServiceException("【车辆预约】:创建订单失败");
        }
        orderResponse.setOrderNo(orderNo);
        orderResponse.setSessionId(IdUtils.randomUUIDByDigit(8));
        asDevice.setStatus(ServiceConstants.VEHICLE_STATUS_IN_APPOINTMENT);
        int u = asDeviceMapper.updateAsDevice(asDevice);
        if(u==0){
            throw new ServiceException("【车辆预约】:更新车辆状态失败");
        }
        return orderResponse;
    }

    /**
     * 取消预约
     * 1.发送  开锁命令
     * 2.更新车辆状态  status 为 ServiceConstants.VEHICLE_STATUS_NORMAL   开锁状态为  开
     * 3.更新订单状态 status 为 ServiceConstants.ORDER_STATUS_CANCEL_APPOINTMENT  记录预约结束时间，计算并更新预约费
     */
    @Transactional
    @SneakyThrows
    @Override
    public OrderResponse cancelAppointment(EtOrderVo appointmentVo, AsDevice asDevice) {
        OrderResponse orderResponse = new OrderResponse();
        String orderNo = appointmentVo.getOrderNo();
        EtOrder order = etOrderService.selectEtOrderByOrderNo(orderNo);
        //1.发送开锁命令并更新车辆状态
        String token = Token.getToken();
        sendCommand(asDevice.getMac(), token,IotConstants.COMMAND_CLOSE,"取消预约关锁");
//        //间隔1秒
//        try {
//            Thread.sleep(500);
//        } catch (InterruptedException ex) {
//            ex.printStackTrace();
//        }
//        sendCommand(asDevice.getMac(), token,IotConstants.COMMAND_PLAY8,"取消预约关锁播报");
        /** 5.记录行程*/
        int tripLog = tripLogService.tripLog(order.getOrderNo(),order.getSn(),ServiceConstants.TRIP_LOG_TYPE_UNLOCK_RIDE);
        if(tripLog==0){
            log.info("【套餐取消预约】记录行程失败");
            throw new ServiceException("【套餐取消预约】记录行程失败");
        }
        //更新订单状态 status 为 ServiceConstants.ORDER_STATUS_CANCEL_APPOINTMENT  记录预约结束时间，计算并更新预约费
        order.setAppointmentEndTime(DateUtils.getNowDate());
        //计算预约费
        calculateAppointmentFee(order);
        order.setTotalFee(order.getAppointmentFee());
        order.setStatus(ServiceConstants.ORDER_STATUS_CANCEL_APPOINTMENT);
        int i = etOrderService.updateEtOrder(order);
        if(i==0){
            throw new ServiceException("【取消预约】:更新订单失败");
        }
        asDevice.setStatus(ServiceConstants.VEHICLE_STATUS_NORMAL);
        asDevice.setLockStatus(ServiceConstants.LOCK_STATUS_OPEN);
        int device = asDeviceMapper.updateAsDevice(asDevice);
        if(device==0){
            log.error("【取消预约】更新车辆状态失败");
            throw new ServiceException("【取消预约】更新车辆状态失败");
        }
        orderResponse.setOrderNo(orderNo);
        orderResponse.setSessionId(IdUtils.randomUUIDByDigit(8));
        return orderResponse;
    }

    /**
     * 计算预约费
     * @param order
     */
    private void calculateAppointmentFee(EtOrder order) {
        int minutes = DateUtils.timeDifferenceInMinutes(order.getAppointmentEndTime(), order.getAppointmentStartTime());//预约分钟数
        //minutes   取模
        if(minutes==0){
            minutes = 1;
        }
        BigDecimal divide = new BigDecimal(minutes).divide(new BigDecimal(10));
        BigDecimal result = divide.setScale(0, RoundingMode.UP);
        //BigDecimal divide 取整 有小数点则+1
        EtOperatingArea area = etOperatingAreaService.selectEtOperatingAreaByAreaId(order.getAreaId());
        BigDecimal appointmentServiceFee = area.getAppointmentServiceFee();
        Integer timeoutMinutes = area.getTimeoutMinutes();
        if(minutes>timeoutMinutes){
            //预约分钟数超过最多保留时间
            throw new ServiceException("预约时间超过最多保留分钟");
        }else{
            //根据开始时间和结束时间计算费用  预约费 = (预约结束时间 - 开始时间) * 预约服务费
            BigDecimal appointmentFee = appointmentServiceFee.multiply(result);
            log.info("【计算预约费】：预约分钟数："+minutes+"，取整："+result+"，预约费："+appointmentFee);
            order.setAppointmentFee(appointmentFee);
        }
    }


    /**
     * 还车
     */
    @SneakyThrows
    @Override
    @Transactional
    public Boolean returnVehicle(String orderNo,String returnType) {
        if(StrUtil.isNotBlank(orderNo)){
            EtOrder order = etOrderService.selectEtOrderByOrderNo(orderNo);
            if(ObjectUtil.isNull(order)){
                throw new ServiceException("订单不存在");
            }
            if(!ServiceConstants.ORDER_TYPE_RIDING.equals(order.getType())){
                throw new ServiceException("该订单非骑行订单");
            }
            if(!ServiceConstants.ORDER_STATUS_RIDING.equals(order.getStatus())){
                throw new ServiceException("该订单状态非骑行中");
            }
            //断是否在禁停区内，如果在禁停区内，不能还车。  noParkingArea
            if(isNoParkingArea(order.getSn(), order.getAreaId())){
                throw new ServiceException("在禁停区内，不能还车");
            }
            /** 1. 记录还车时间*/
            order.setReturnType(returnType);
            order.setStatus(ServiceConstants.ORDER_STATUS_RIDING_END);
            order.setReturnTime(DateUtils.getNowDate());
            String token = Token.getToken();
            AsDevice device = asDeviceMapper.selectAsDeviceBySn(order.getSn());
            /** 2. 车辆远程关锁*/
            sendCommand(device.getMac(), token,IotConstants.COMMAND_CLOSE,"还车关锁");
            //间隔1秒
//            try {
//                Thread.sleep(500);
//            } catch (InterruptedException ex) {
//                ex.printStackTrace();
//            }
//            sendCommand(device.getMac(), token,IotConstants.COMMAND_PLAY8,"还车关锁播报");
            /** 4. 更新车辆状态*/
            device.setStatus(ServiceConstants.VEHICLE_STATUS_NORMAL);
            device.setLockStatus(ServiceConstants.LOCK_STATUS_CLOSE);
            int deviceUpdate = asDeviceMapper.updateAsDevice(device);
            if(deviceUpdate==0){
                log.info("【还车关锁】更新车辆状态失败");
                throw new ServiceException("【还车关锁】更新车辆状态失败");
            }
            /** 3. 计算订单费用，保存订单总金额*/
            order = calculateOrderFee(order);
            int i = etOrderService.updateEtOrder(order);
            if(i==0){
                throw new ServiceException("更新订单状态失败");
            }
            /** 5.记录行程*/
            int tripLog = tripLogService.tripLog(order.getOrderNo(),device.getSn(),ServiceConstants.TRIP_LOG_TYPE_RETRUN_LOCK);
            if(tripLog==0){
                log.info("【还车关锁】记录行程失败");
                throw new ServiceException("【还车关锁】记录行程失败");
            }
            log.info("还车成功");
            return Boolean.TRUE;
        }else{
            throw new ServiceException("订单号不能为空");
        }
    }

    /**
     * 计算订单费用
     */
    private EtOrder calculateOrderFee(EtOrder order) {
        String type = order.getType();
        if(type.equals("1")){//骑行订单,正常骑行（包含预约费），计时收费，根据开锁时间、起步价和起步时长、时长费和时长分钟等参数，如果有预约，还要加上预约费计算费用,还要判断封顶费用（根据开锁时间）
//            /** 预约费*/
//            Date appointmentStartTime = order.getAppointmentStartTime();
//            if(ObjectUtils.isNotEmpty(appointmentStartTime)){
//                //有预约，根据开始时间和结束时间计算费用  预约费 = (预约结束时间 - 开始时间) * 预约服务费
//                calculateAppointmentFee(order);
//            }
            Long ruleId = order.getRuleId();
            EtFeeRule rule = etFeeRuleService.selectEtFeeRuleByRuleId(ruleId);
            String rentalUnit = rule.getRentalUnit();//租赁单位

            //根据订单的开始时间和还车时间计算出在第几个计费周期，在第一个计费周期内，并没有超过封顶费用，则根据规则算出骑行费，
            Integer cycle = null;
            Date startTime = order.getUnlockTime();
            Date endTime = order.getReturnTime();
            String chargingCycle = rule.getChargingCycle();
            String chargingCycleValue = rule.getChargingCycleValue();

            if(ServiceConstants.CHARGING_CYCLE_HOUR.equals(chargingCycle)) {//订单生成后__小时  第几个
                cycle = inHowManyChargingCycle(startTime, endTime, chargingCycleValue);
                BigDecimal ridingFee;
                if(cycle == 1){
                    order.setCycle(1);//在第几个计费周期内
                    ridingFee = calculateRidingFee(order.getUnlockTime(), order, rule, rule.getRidingRuleJson(), rentalUnit);
                }else {
                    // 骑行费 = (第几个周期 - 1) *  封顶费用 + 骑行费
                    // 算出最后一个周期的开始时间
                    order.setCycle(cycle);
                    Date afterXHours = DateUtils.getTimeAfterXHours(order.getUnlockTime(), Integer.parseInt(chargingCycleValue)*(cycle-1));
                    ridingFee = calculateRidingFee(afterXHours, order, rule, rule.getRidingRuleJson(),rentalUnit);
                }
                Boolean comparisonResult = ridingFee.compareTo(rule.getCappedAmount()) >= 0;//比较订单的骑行费+预约费是否大于等于封顶费用
                if(comparisonResult){//超出封顶费用按封顶费用计算
                    order.setRidingFee(rule.getCappedAmount());
                }else{
                    order.setRidingFee(ridingFee);
                }
            }else if(ServiceConstants.CHARGING_CYCLE_CUSTOM.equals(chargingCycle)){//自定义时刻
                // todo 自定义计费周期时刻  获取到自定义时刻，判断骑行的这段时间是否在自定义时刻里    ____.___.___|___________|
                // 如果不在，

            }

            EtOperatingArea area = etOperatingAreaService.selectEtOperatingAreaByAreaId(order.getAreaId());
            /** 管理费*/
            // 根据车辆的定位判断是否在停车区内
            if(!isParkingZone(order.getSn(), order.getAreaId())){
                BigDecimal vehicleManagementFee = area.getVehicleManagementFee();//车辆管理费
                order.setManageFee(vehicleManagementFee);
            }
            /** 调度费*/
            //判断是否在运营区内
            if(!isAreaZone(order.getSn(), area)){
                BigDecimal dispatchFee = area.getDispatchFee();//调度费
                order.setDispatchFee(dispatchFee);
            }
            BigDecimal totalFee = BigDecimal.ZERO;
            if (order != null) {
                BigDecimal ridingFee;
                // 确保所有费用都是BigDecimal.ZERO，如果它们没有被设置
                if (order.getManageFee() == null) {
                    order.setManageFee(BigDecimal.ZERO);
                }
                if (order.getDispatchFee() == null) {
                    order.setDispatchFee(BigDecimal.ZERO);
                }
                if (order.getAppointmentFee() == null) {
                    order.setAppointmentFee(BigDecimal.ZERO);
                }
                if (order.getRidingFee() == null || order.getRidingFee().compareTo(BigDecimal.ZERO) == 0) {
                    order.setRidingFee(BigDecimal.ZERO);
                    totalFee = BigDecimal.ZERO;//如果骑行费是0，说明在免费骑行时间内，那订单的总金额也是0
                }else{
                    ridingFee = new BigDecimal(cycle - 1).multiply(rule.getCappedAmount()).add(order.getRidingFee());
                    totalFee = ridingFee.add(order.getManageFee()).add(order.getDispatchFee()).add(order.getAppointmentFee());
                }
            }
            log.info("【计算订单费用】骑行费：" + order.getRidingFee() + "，车辆管理费：" + order.getManageFee() + "，调度费：" + order.getDispatchFee() + "，预约费：" + order.getAppointmentFee() + "，总计：" + totalFee);
            order.setTotalFee(totalFee);
        }else {
            throw new ServiceException("订单类型错误");
        }
        return order;
    }



    /**
     * 计算骑行费
     */
    @NotNull
    private BigDecimal calculateRidingFee(Date unlockTime, EtOrder order, EtFeeRule rule, String ridingRuleJson,String rentalUnit) {
        BigDecimal ridingFee = BigDecimal.ZERO;
        //没有预约，根据订单中的开锁时间计算费用：骑行费用： ((还车时间-开锁时间转成分钟数)/ 时长分钟)-起步价分钟数 * 时长费 + 起步价
        Date returnTime = order.getReturnTime();//还车时间

        //在免费骑行时间内，骑行费为0
        Integer minutes = DateUtils.timeDifferenceInMinutes(returnTime,unlockTime);//计算相隔多少分钟
        Integer freeRideTime = rule.getFreeRideTime();
        if(freeRideTime!=0 && freeRideTime!=null && minutes<=freeRideTime){
            return ridingFee;
        }

        if(rule.getRidingRule().equals(ServiceConstants.RIDING_RULE_STARTING_PRICE_MODE)){//1-起步价计费
            StartingRuleVo startingRule = JSONObject.parseObject(ridingRuleJson, StartingRuleVo.class);
            String startingPrice = startingRule.getStartingPrice();//起步价
            Integer startingTime = Integer.parseInt(startingRule.getStartingTime());//起步时间
            String timeoutPrice = startingRule.getTimeoutPrice();//超时价格
            Integer timeoutTime = Integer.parseInt(startingRule.getTimeoutTime());//超时时间

            //判断免费骑行时间不得大于起步时间
            if(freeRideTime>startingTime){
                throw new ServiceException("免费骑行时长不得大于起步分钟！");
            }

            if(rentalUnit.equals(ServiceConstants.RENTAL_UNIT_HOURS)){//租赁单位：hours-小时
                startingTime = startingTime*60;
                timeoutTime = timeoutTime*60;
            }else if(rentalUnit.equals(ServiceConstants.RENTAL_UNIT_DAY)){//租赁单位：day-天
                startingTime = startingTime*60*24;
                timeoutTime = timeoutTime*60*24;
            }

            if(ObjectUtils.isNotEmpty(returnTime) && ObjectUtils.isNotEmpty(unlockTime)){
                if(minutes<startingTime){
                    //小于起步分钟，按照起步价计算费用
                    ridingFee = new BigDecimal(startingPrice);
                }else{
                    //大于起步分钟，按照时长费计算费用
                    //minutes除以startingMinutes得到的结构四舍五入
                    minutes = minutes - timeoutTime;//扣掉起步分钟数后
                    double ceil = Math.ceil(minutes / timeoutTime) +1 ;
                    ridingFee = new BigDecimal(ceil * Integer.parseInt(timeoutPrice));
                    BigDecimal startingPriceB = new BigDecimal(startingPrice);
                    ridingFee = ridingFee.add(startingPriceB);
                }
            }
        }else{//2-区间计费
            IntervalRuleVo intervalRuleVo = JSONObject.parseObject(ridingRuleJson, IntervalRuleVo.class);
            List<IntervalRuleVo.IntervalRule> rule1 = intervalRuleVo.getRule();
            //骑行时长
            Integer duration = DateUtils.timeDifferenceInMinutes(returnTime,unlockTime);//计算相隔多少分钟
            BigDecimal durationInMinutes = new BigDecimal(duration);
            if(rentalUnit.equals(ServiceConstants.RENTAL_UNIT_HOURS)){//租赁单位：hours-小时
                BigDecimal durationInHours = durationInMinutes.divide(BigDecimal.valueOf(60), 0, RoundingMode.CEILING);
                duration = durationInHours.setScale(0, RoundingMode.CEILING).intValue();
            }else if(rentalUnit.equals(ServiceConstants.RENTAL_UNIT_DAY)){//租赁单位：day-天
                BigDecimal durationInDays = durationInMinutes.divide(BigDecimal.valueOf(60*24), 0, RoundingMode.CEILING); // 转换为天数并向上取整
                duration = durationInDays.setScale(0, RoundingMode.CEILING).intValue(); // 转换为整数天数
            }
            //判断在哪个计费区间内，
            for (IntervalRuleVo.IntervalRule intervalRule : rule1) {
                Integer start = intervalRule.getStart();
                Integer end = intervalRule.getEnd();
                Integer unit = intervalRule.getEachUnit();
                BigDecimal fee = intervalRule.getFee();
                // 判断duration是否在当前区间内
                if (duration >= start && duration < end) {
                    // duration位于当前区间[start, end]内
                    System.out.println("费用为: " + fee + "元/每" + unit + formatUnit(rentalUnit));
                    ridingFee = new BigDecimal(fee.toString()).multiply(new BigDecimal(duration / unit));
                } else if (duration == end && intervalRule.getEnd() != 9999) {
                    // 特殊处理结束边界为具体值的情况，避免与默认无限大区间混淆
                    System.out.println("达到区间的上限，但不落入下一个区间，费用为: " + fee + "元/每" + unit + formatUnit(rentalUnit));
                    throw new ServiceException("达到区间的上限，但不落入下一个区间，费用为: " + fee + "元/每" + unit + formatUnit(rentalUnit));
                }
            }
        }
        return ridingFee;
    }

    /**
     * 格式化单位
     */
    private String formatUnit(String rentalUnit) {
        if(ServiceConstants.RENTAL_UNIT_MINUTES.equals(rentalUnit)){
            return "分钟";
        }else if(ServiceConstants.RENTAL_UNIT_HOURS.equals(rentalUnit)){
            return "小时";
        } else if(ServiceConstants.RENTAL_UNIT_DAY.equals(rentalUnit)){
            return "天";
        } else {
            return "";
        }
    }


    /**
     * 计算出在第几个计费周期中
     */
    public Integer inHowManyChargingCycle(Date startTime, Date endTime,String chargingCycleValue) {
        // 计算两个时间的毫秒差
        long diffInMillies = endTime.getTime() - startTime.getTime();

        // 将毫秒差转换为小时
        long hours = TimeUnit.MILLISECONDS.toHours(diffInMillies);

        if(hours==0){
            return 1;
        }
        // 计算出在第几个计费周期中
        BigDecimal cycle = BigDecimal.valueOf(hours).divide(new BigDecimal(chargingCycleValue), 0, RoundingMode.CEILING);
        return cycle.intValue();
    }

    /**
     * 是否在停车区内
     */
    private Boolean isParkingZone(String sn,Long areaId) {
        Boolean inCircle = false;
        EtParkingArea parkingArea = new EtParkingArea();
        parkingArea.setAreaId(areaId);
        List<EtParkingArea> parkingAreas = parkingAreaService.selectEtParkingAreaList(parkingArea);
        EtOperatingArea area = etOperatingAreaService.selectEtOperatingAreaByAreaId(areaId);
        if(ObjectUtil.isNull(parkingAreas) || parkingAreas.size() == 0){
            log.info("运营区【{}】没有停车区,",areaId);
            throw new ServiceException("运营区【{}】没有停车区"+areaId.toString());
        }
        double tolerance = area.getError(); // 误差距离
        for (EtParkingArea etParkingArea : parkingAreas) {
            if(etParkingArea.getType().equals(ServiceConstants.PARKING_AREA_TYPE_PARKFING)){
                AsDevice device = asDeviceMapper.selectAsDeviceBySn(sn);
                String latitude = device.getLatitude();
                String longitude = device.getLongitude();
                Geometry geometry = GeoUtils.fromWkt(etParkingArea.getBoundary());
                inCircle = GeoUtils.isInPolygonWithTolerance(longitude, latitude, geometry, tolerance);
                if(inCircle){
                    log.info("车辆【{}】在停车区【{}】内",sn,etParkingArea.getParkingName());
                    inCircle = true;
                    break;
                }else{
                    log.info("车辆【{}】不在停车区【{}】内",sn,etParkingArea.getParkingName());
                }
            }
        }
        return inCircle;
    }

    /**
     * 是否在运营区内
     */
    @Override
    public Boolean isAreaZone(String sn,EtOperatingArea area) {
        Boolean inCircle  = false;
        AsDevice device = asDeviceMapper.selectAsDeviceBySn(sn);
        String latitude = device.getLatitude();
        String longitude = device.getLongitude();
        Geometry geometry = GeoUtils.fromWkt(area.getBoundary());
        inCircle = GeoUtils.isInCircle(longitude, latitude, geometry);
        if(inCircle){
            log.info("车辆【{}】在运营区【{}】内",sn,area.getAreaName());
            inCircle = true;
        }else{
            log.info("车辆【{}】不在运营区【{}】内",sn,area.getAreaName());
        }
        return inCircle;
    }

    /**
     * 是否禁停区内
     */
    public boolean isNoParkingArea(String sn,Long areaId) {
        Boolean inCircle = false;
        EtParkingArea parkingArea = new EtParkingArea();
        parkingArea.setAreaId(areaId);
        List<EtParkingArea> parkingAreas = parkingAreaService.selectEtParkingAreaList(parkingArea);
        if(ObjectUtil.isNull(parkingAreas) || parkingAreas.size() == 0){
            log.info("运营区【{}】没有禁停区,",areaId);
            throw new ServiceException("运营区【{}】没有禁停区"+areaId.toString());
        }
        for (EtParkingArea etParkingArea : parkingAreas) {
            if(etParkingArea.getType().equals(ServiceConstants.PARKING_AREA_TYPE_NO_PARKFING)){
                AsDevice device = asDeviceMapper.selectAsDeviceBySn(sn);
                String latitude = device.getLatitude();
                String longitude = device.getLongitude();
                Geometry geometry = GeoUtils.fromWkt(etParkingArea.getBoundary());
                inCircle = GeoUtils.isInCircle(longitude, latitude, geometry);
                if(inCircle){
                    log.info("车辆【{}】在禁停区【{}】内",sn,etParkingArea.getParkingName());
                    inCircle = true;
                    break;
                }else{
                    log.info("车辆【{}】不在禁停区【{}】内",sn,etParkingArea.getParkingName());
                }
            }
        }
        return inCircle;
    }

    /**
     * 是否禁行区内
     */
    @Override
    public boolean isNoRidingArea(String sn,Long areaId) {
        Boolean isNoRiding = false;
        EtParkingArea parkingArea = new EtParkingArea();
        parkingArea.setAreaId(areaId);
        List<EtParkingArea> parkingAreas = parkingAreaService.selectEtParkingAreaList(parkingArea);
        if(ObjectUtil.isNull(parkingAreas) || parkingAreas.size() == 0){
            log.info("运营区【{}】没有禁行区,",areaId);
            throw new ServiceException("运营区【{}】没有禁行区"+areaId.toString());
        }
        for (EtParkingArea etParkingArea : parkingAreas) {
            if(etParkingArea.getType().equals(ServiceConstants.PARKING_AREA_TYPE_BANNED_RIDING)){
                AsDevice device = asDeviceMapper.selectAsDeviceBySn(sn);
                String latitude = device.getLatitude();
                String longitude = device.getLongitude();
                Geometry geometry = GeoUtils.fromWkt(etParkingArea.getBoundary());
                isNoRiding = GeoUtils.isInCircle(longitude, latitude, geometry);
                if(isNoRiding){
                    log.info("车辆【{}】在禁行区【{}】内",sn,etParkingArea.getParkingName());
                    isNoRiding = true;
                    break;
                }else{
                    log.info("车辆【{}】不在禁行区【{}】内",sn,etParkingArea.getParkingName());
                }
            }
        }
        return isNoRiding;
    }

    /**
     * 是否低电量
     */
    @Override
    public boolean isLowBattery(String sn) {
        AsDevice device = asDeviceMapper.selectAsDeviceBySn(sn);
        String remainingPower = device.getRemainingPower();
        log.info("车辆【{}】电量为【{}】",sn,remainingPower);
        EtOperatingArea area = etOperatingAreaService.selectEtOperatingAreaByAreaId(device.getAreaId());
        if(Integer.parseInt(remainingPower) < Integer.parseInt(area.getUndercharge())){
            log.info("车辆【{}】电量为【{}】低于【{}】",sn,remainingPower,area.getUndercharge());
            return true;
        }
        return false;
    }

    /**
     * 判断是否在运营时间内
     */
    @Override
    public Boolean isOperatingTime(String sn) {
        AsDevice device = asDeviceMapper.selectAsDeviceBySn(sn);
        EtOperatingArea area = etOperatingAreaService.selectEtOperatingAreaByAreaId(device.getAreaId());
        if(area.getAreaTime().equals(ServiceConstants.AREA_TIME_CUSTOM) && !DateUtils.isInTime(area.getAreaTimeStart(),area.getAreaTimeEnd())){
            log.info("不在运营时间内，运营时间为：【{}】--【{}】",area.getAreaTimeStart(),area.getAreaTimeEnd());
            return Boolean.FALSE;
        }
        return Boolean.TRUE;
    }

    /**
     * 判断是否靠近边界
     */
//    @Override
//    public boolean isCloseToTheBoundary(String sn, EtOperatingArea area) {
//        Boolean isClose = false;
//        AsDevice device = asDeviceMapper.selectAsDeviceBySn(sn);
//        String latitude = device.getLatitude();
//        String longitude = device.getLongitude();
//        Geometry geometry = GeoUtils.fromWkt(area.getBoundary());
//
//        double distanceToBoundary = GeoUtils.distanceToBoundary(longitude, latitude, geometry);
//        if(distanceToBoundary <= 3.0) {
//            log.info("车辆【{}】靠近【{}】运营边界",sn,area.getAreaName());
//            isClose = true;
//        }else{
//            log.info("车辆【{}】为靠近靠近【{}】运营边界",sn,area.getAreaName());
//        }
//        return isClose;
//    }


}
