package com.ruoyi.system.service.impl;

import java.util.List;
import com.ruoyi.common.utils.DateUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;
import com.ruoyi.system.mapper.EtCapitalFlowMapper;
import com.ruoyi.system.domain.EtCapitalFlow;
import com.ruoyi.system.service.IEtCapitalFlowService;

/**
 * 资金流水Service业务层处理
 *
 * @author 邱贞招
 * @date 2024-05-24
 */
@Service
public class EtCapitalFlowServiceImpl implements IEtCapitalFlowService
{
    @Autowired
    private EtCapitalFlowMapper etCapitalFlowMapper;

    /**
     * 查询资金流水
     *
     * @param flowId 资金流水主键
     * @return 资金流水
     */
    @Override
    public EtCapitalFlow selectEtCapitalFlowByFlowId(Long flowId)
    {
        return etCapitalFlowMapper.selectEtCapitalFlowByFlowId(flowId);
    }

    /**
     * 根据第三方交易单号查询资金流水
     *
     * @param outTradeNo 第三方交易单号
     * @return 资金流水
     */
    @Override
    public EtCapitalFlow selectEtCapitalFlowByOutTradeNo(String outTradeNo) {
        return etCapitalFlowMapper.selectEtCapitalFlowByOutTradeNo(outTradeNo);
    }

    /**
     * 查询资金流水列表
     *
     * @param etCapitalFlow 资金流水
     * @return 资金流水
     */
    @Override
    public List<EtCapitalFlow> selectEtCapitalFlowList(EtCapitalFlow etCapitalFlow)
    {
        return etCapitalFlowMapper.selectEtCapitalFlowList(etCapitalFlow);
    }

    /**
     * 新增资金流水
     *
     * @param etCapitalFlow 资金流水
     * @return 结果
     */
    @Override
    public int insertEtCapitalFlow(EtCapitalFlow etCapitalFlow)
    {
        etCapitalFlow.setCreateTime(DateUtils.getNowDate());
        return etCapitalFlowMapper.insertEtCapitalFlow(etCapitalFlow);
    }

    /**
     * 修改资金流水
     *
     * @param etCapitalFlow 资金流水
     * @return 结果
     */
    @Override
    public int updateEtCapitalFlow(EtCapitalFlow etCapitalFlow)
    {
        return etCapitalFlowMapper.updateEtCapitalFlow(etCapitalFlow);
    }

    /**
     * 批量删除资金流水
     *
     * @param flowIds 需要删除的资金流水主键
     * @return 结果
     */
    @Override
    public int deleteEtCapitalFlowByFlowIds(Long[] flowIds)
    {
        return etCapitalFlowMapper.deleteEtCapitalFlowByFlowIds(flowIds);
    }

    /**
     * 删除资金流水信息
     *
     * @param flowId 资金流水主键
     * @return 结果
     */
    @Override
    public int deleteEtCapitalFlowByFlowId(Long flowId)
    {
        return etCapitalFlowMapper.deleteEtCapitalFlowByFlowId(flowId);
    }
}
