package com.ruoyi.system.service.impl;

import java.util.List;
import com.ruoyi.common.utils.DateUtils;
import com.ruoyi.system.service.IAsUserService;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;
import com.ruoyi.system.mapper.EtFeeRuleMapper;
import com.ruoyi.system.domain.EtFeeRule;
import com.ruoyi.system.service.IEtFeeRuleService;

/**
 * 收费方式Service业务层处理
 *
 * @author 邱贞招
 * @date 2024-04-08
 */
@Service
public class EtFeeRuleServiceImpl implements IEtFeeRuleService
{
    @Autowired
    private EtFeeRuleMapper etFeeRuleMapper;

    /**
     * 查询收费方式
     *
     * @param ruleId 收费方式主键
     * @return 收费方式
     */
    @Override
    public EtFeeRule selectEtFeeRuleByRuleId(Long ruleId)
    {
        return etFeeRuleMapper.selectEtFeeRuleByRuleId(ruleId);
    }

    /**
     * 查询收费方式列表
     *
     * @param etFeeRule 收费方式
     * @return 收费方式
     */
    @Override
    public List<EtFeeRule> selectEtFeeRuleList(EtFeeRule etFeeRule)
    {
        return etFeeRuleMapper.selectEtFeeRuleList(etFeeRule);
    }

    /**
     * 根据运营区获取收费方式列表
     *
     * @param areaId 运营区id
     * @return 结果
     */
    @Override
    public List<EtFeeRule> selectEtFeeRuleListByAreaId(Long areaId) {
        return etFeeRuleMapper.selectRuleInfoListByAreaId(areaId);
    }

    /**
     * 新增收费方式
     *
     * @param etFeeRule 收费方式
     * @return 结果
     */
    @Override
    public int insertEtFeeRule(EtFeeRule etFeeRule)
    {
        etFeeRule.setCreateTime(DateUtils.getNowDate());
        etFeeRule.setStatus("0");
        return etFeeRuleMapper.insertEtFeeRule(etFeeRule);
    }

    /**
     * 修改收费方式
     *
     * @param etFeeRule 收费方式
     * @return 结果
     */
    @Override
    public int updateEtFeeRule(EtFeeRule etFeeRule)
    {
        return etFeeRuleMapper.updateEtFeeRule(etFeeRule);
    }

    /**
     * 批量删除收费方式
     *
     * @param ruleIds 需要删除的收费方式主键
     * @return 结果
     */
    @Override
    public int deleteEtFeeRuleByRuleIds(Long[] ruleIds)
    {
        return etFeeRuleMapper.deleteEtFeeRuleByRuleIds(ruleIds);
    }

    /**
     * 删除收费方式信息
     *
     * @param ruleId 收费方式主键
     * @return 结果
     */
    @Override
    public int deleteEtFeeRuleByRuleId(Long ruleId)
    {
        return etFeeRuleMapper.deleteEtFeeRuleByRuleId(ruleId);
    }

    /**
     * 根据区域ID获取收费方式选择框列表
     *
     * @param areaId 用户ID
     * @return 选中收费方式ID列表
     */
    @Override
    public List<Long> selectRuleListByAreaId(Long areaId) {
        return etFeeRuleMapper.selectRuleListByAreaId(areaId);
    }

    /**
     * 根据区域ID获取收费方式名称选择框列表
     *
     * @param areaId 用户ID
     * @return 选中收费方式名称列表
     */
    @Override
    public List<String> selectRuleNameListByAreaId(Long areaId) {
        return etFeeRuleMapper.selectRuleNameListByAreaId(areaId);
    }
}
