package com.ruoyi.system.service.impl;

import java.util.List;

import cn.hutool.core.util.ObjectUtil;
import cn.hutool.core.util.StrUtil;
import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.ruoyi.common.utils.DateUtils;
import com.ruoyi.system.domain.AsDevice;
import com.ruoyi.system.mapper.AsDeviceMapper;
import com.ruoyi.system.service.IAsDeviceService;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;
import com.ruoyi.system.mapper.EtModelMapper;
import com.ruoyi.system.domain.EtModel;
import com.ruoyi.system.service.IEtModelService;

import javax.annotation.Resource;

/**
 * 车辆型号Service业务层处理
 *
 * @author 邱贞招
 * @date 2024-05-13
 */
@Service
public class EtModelServiceImpl implements IEtModelService
{
    @Autowired
    private EtModelMapper etModelMapper;

    @Autowired
    private IAsDeviceService asDeviceService;

    /**
     * 查询车辆型号
     *
     * @param modelId 车辆型号主键
     * @return 车辆型号
     */
    @Override
    public EtModel selectEtModelByModelId(Long modelId)
    {
        EtModel etModel = etModelMapper.selectEtModelByModelId(modelId);
        if(ObjectUtil.isNotNull(etModel)){
            Integer allNum = asDeviceService.selectCountByModelId(modelId);
            etModel.setDeviceNum(allNum);
        }
        return etModel;
    }

    /**
     * 查询车辆型号列表  合伙人  Partner
     *
     * @param etModel 车辆型号
     * @return 车辆型号
     */
    @Override
    public List<EtModel> selectEtModelList(EtModel etModel)
    {
        List<EtModel> etModels = etModelMapper.selectEtModelList(etModel);
        for(EtModel model: etModels){
            AsDevice device = new AsDevice();
            device.setModelId(model.getModelId());
            List<AsDevice> asDevices = asDeviceService.selectAsDeviceList(device);
            model.setDeviceNum(asDevices.size());
        }
        return etModels;
    }

    /**
     * 新增车辆型号
     *
     * @param etModel 车辆型号
     * @return 结果
     */
    @Override
    public int insertEtModel(EtModel etModel)
    {
        etModel.setCreateTime(DateUtils.getNowDate());
        return etModelMapper.insertEtModel(etModel);
    }

    /**
     * 修改车辆型号
     *
     * @param etModel 车辆型号
     * @return 结果
     */
    @Override
    public int updateEtModel(EtModel etModel)
    {
        etModel.setUpdateTime(DateUtils.getNowDate());
        return etModelMapper.updateEtModel(etModel);
    }

    /**
     * 批量删除车辆型号
     *
     * @param modelIds 需要删除的车辆型号主键
     * @return 结果
     */
    @Override
    public int deleteEtModelByModelIds(Long[] modelIds)
    {
        return etModelMapper.deleteEtModelByModelIds(modelIds);
    }

    /**
     * 删除车辆型号信息
     *
     * @param modelId 车辆型号主键
     * @return 结果
     */
    @Override
    public int deleteEtModelByModelId(Long modelId)
    {
        return etModelMapper.deleteEtModelByModelId(modelId);
    }

}
