package com.ruoyi.system.service.impl;

import cn.hutool.core.util.ObjectUtil;
import cn.hutool.core.util.StrUtil;
import com.ruoyi.common.constant.Constants;
import com.ruoyi.common.constant.IotConstants;
import com.ruoyi.common.constant.ServiceConstants;
import com.ruoyi.common.core.domain.entity.AsUser;
import com.ruoyi.common.core.redis.RedisCache;
import com.ruoyi.common.exception.ServiceException;
import com.ruoyi.common.utils.CommonUtil;
import com.ruoyi.common.utils.DateUtils;
import com.ruoyi.common.utils.onenet.Token;
import com.ruoyi.common.utils.uuid.IdUtils;
import com.ruoyi.system.domain.*;
import com.ruoyi.system.domain.vo.*;
import com.ruoyi.system.mapper.AsDeviceMapper;
import com.ruoyi.system.mapper.EtOrderMapper;
import com.ruoyi.system.service.*;
import com.wechat.pay.java.service.payments.jsapi.model.PrepayWithRequestPaymentResponse;
import com.wechat.pay.java.service.refund.model.Refund;
import lombok.SneakyThrows;
import lombok.extern.slf4j.Slf4j;
import org.apache.commons.lang3.ObjectUtils;
import org.springframework.beans.BeanUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.beans.factory.annotation.Value;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;
import org.springframework.transaction.support.TransactionTemplate;
import org.springframework.web.bind.annotation.RequestBody;

import javax.annotation.Resource;
import java.math.BigDecimal;
import java.text.SimpleDateFormat;
import java.util.*;

/**
 * 订单Service业务层处理
 *
 * @author 邱贞招
 * @date 2024-04-08
 */
@Slf4j
@Service
public class EtOrderServiceImpl implements IEtOrderService
{
    @Resource
    private EtOrderMapper etOrderMapper;

    @Resource
    private AsDeviceMapper asDeviceMapper;

    @Resource
    private IAsUserService asUserService;

    @Autowired
    private IEtOperatingAreaService etOperatingAreaService;

    @Autowired
    private IWxPayService wxPayService;

    @Autowired
    private IAsDeviceService deviceService;

    @Autowired
    private ISysConfigService sysConfigService;

    @Autowired
    private IEtFeeRuleService etFeeRuleService;

    @Autowired
    private IEtOrderService etOrderService;

    @Autowired
    private IAsDeviceService asDeviceService;

    @Autowired
    private TransactionTemplate transactionTemplate;

    @Autowired
    private IEtRefundService etRefundService;

    @Autowired
    private IEtTripLogService etTripLogService;

    @Autowired
    private RedisCache redisCache;

//    @Autowired
//    private TokenService tokenService;

    // 令牌有效期（默认30分钟）
    @Value("${token.expireTime}")
    private int expireTime;



    /**
     * 查询订单
     *
     * @param orderId 订单主键
     * @return 订单
     */
    @Override
    public EtOrder selectEtOrderByOrderId(Long orderId)
    {
        return etOrderMapper.selectEtOrderByOrderId(orderId);
    }

    /**
     * 根据订单号查询订单
     *
     * @param orderNo 订单号
     * @return 订单
     */
    @Override
    public EtOrder selectEtOrderByOrderNo(String orderNo)
    {
        return etOrderMapper.selectEtOrderByOrderNo(orderNo);
    }

    /**
     * 根据微信内部订单号查询订单
     *
     * @param outTradeNo 订单号
     * @return 订单
     */
    @Override
    public EtOrder selectEtOrderByOutTradeNo(String outTradeNo)
    {
        return etOrderMapper.selectEtOrderByOutTradeNo(outTradeNo);
    }

    /**
     * 根据订单号查询订单(详细)
     *
     * @param orderNo 订单号
     * @return 订单
     */
    @Override
    public EtOrder getOrderInfo(String orderNo)
    {
        EtOrder order = etOrderMapper.selectEtOrderByOrderNo(orderNo);
        if (ObjectUtils.isEmpty(order)) {
            throw new ServiceException("订单不存在");
        }
        if (ObjectUtils.isNotEmpty(order)) {
            AsUser user = asUserService.selectUserById(order.getUserId());
            if(ObjectUtils.isNotEmpty(user)){
                order.setPhonenumber(user.getPhonenumber());
                order.setUserName(user.getUserName());
            }
            EtOperatingArea area = etOperatingAreaService.selectEtOperatingAreaByAreaId(order.getAreaId());
            if(ObjectUtils.isNotEmpty(area)){
                order.setArea(area.getAreaName());
            }
            AsDevice device = asDeviceMapper.selectAsDeviceBySn(order.getSn());
            order.setDevice(device);
            //根据规则id查询规则
            EtFeeRule etFeeRule = etFeeRuleService.selectEtFeeRuleByRuleId(order.getRuleId());
            if(ObjectUtils.isNotEmpty(etFeeRule)){
                order.setRule(etFeeRule);
            }
            //退款记录
            EtRefund etRefund = new EtRefund();
            etRefund.setOrderNo(order.getOrderNo());
            List<EtRefund> refundList = etRefundService.selectEtRefundList(etRefund);
            if(ObjectUtils.isNotEmpty(refundList)){
                EtRefund etRefund1 = refundList.get(0);
                order.setEtRefund(etRefund1);
                order.setSettlementFee(order.getTotalFee().subtract(etRefund1.getAmount()));
            }else{
                order.setSettlementFee(order.getTotalFee());
            }
            //行程记录
            EtTripLog tripLog = new EtTripLog();
            tripLog.setOrderNo(order.getOrderNo());
            List<EtTripLog> tripLogs = etTripLogService.selectEtTripLogList(tripLog);
            if(ObjectUtils.isNotEmpty(tripLogs)){
                order.setTripLogs(tripLogs);
            }
        }
        return order;
    }

    /**
     * 查询订单列表
     *  车型
     *  	品牌商
     *  运营商
     *  满电电压  Full voltage
     *  亏电电压  Low voltage
     *  满电续航  full_endurance
     *  低电量提醒  Low battery reminder
     *  已投放区域
     * @param etOrder 订单
     * @return 订单  已结束
     */
    @Override
    public List<EtOrder> selectEtOrderList(EtOrder etOrder)
    {
        String[] statusList = etOrder.getStatusList();
        if(ObjectUtil.isNotNull(statusList) && toBePaid(statusList)){
            etOrder.setPaid("0");
        }
        if((ObjectUtil.isNotNull(statusList) && ended(statusList)) || ServiceConstants.ORDER_TYPE_DEPOSIT.equals(etOrder.getType()) ){
            etOrder.setPaid("1");
        }
        List<EtOrder> etOrders = etOrderMapper.selectEtOrderList(etOrder);
        //如果查询押金则增加退款记录
        if(ServiceConstants.ORDER_TYPE_DEPOSIT.equals(etOrder.getType())){
            etOrders.forEach(etOrder1 -> {
                EtRefund refund = etRefundService.selectEtRefundByOrderNo(etOrder1.getOrderNo());
                if(ObjectUtil.isNotNull(refund)){
                    etOrder1.setEtRefund(refund);
                }
            });
        }
        return etOrders;
    }

    /**
     * 查询充值/退款订单列表  资本 capital flow  收支  业务
     */
    @Override
    public List<RechargeVo> rechargeList(EtOrder etOrder) {
        List<RechargeVo> rechargeVoList = new ArrayList<>();
        etOrder.setPaid(ServiceConstants.ORDER_PAY_STATUS_PAID);
        etOrder.setStatus(ServiceConstants.ORDER_STATUS_ORDER_END);
        List<EtOrder> etOrders = etOrderMapper.selectEtOrderList(etOrder);
        etOrders.forEach(etOrder1 -> {
            RechargeVo rechargeVo = new RechargeVo();
            BeanUtils.copyProperties(etOrder1,rechargeVo);
            rechargeVo.setRechargeStatus(ServiceConstants.RECHARGE_STATUS_SUCCESS);
            rechargeVo.setRechargeType(ServiceConstants.ORDER_TYPE_DEPOSIT);
            rechargeVoList.add(rechargeVo);
        });
        //如果查询押金则增加退款记录
        EtRefund refund = new EtRefund();
        refund.setType(ServiceConstants.REFUND_TYPE_DEPOSIT);
        refund.setRefundResult(Constants.SUCCESS2);
        List<EtRefund> etRefunds = etRefundService.selectEtRefundList(refund);
        etRefunds.forEach(etRefund -> {
            AsUser asUser = asUserService.selectUserById(etRefund.getUserId());
            RechargeVo rechargeVo = new RechargeVo();
            BeanUtils.copyProperties(etRefund,rechargeVo);
            rechargeVo.setOutTradeNo(etRefund.getRefundNo());
            rechargeVo.setPayTime(etRefund.getCreateTime());
            rechargeVo.setTotalFee(etRefund.getAmount());
            rechargeVo.setUserId(etRefund.getUserId());
            rechargeVo.setUserName(asUser.getUserName());
            rechargeVo.setPhonenumber(asUser.getPhonenumber());
            rechargeVo.setRechargeStatus(ServiceConstants.RECHARGE_STATUS_REFUND_SUCCESS);
            rechargeVo.setRechargeType(ServiceConstants.ORDER_TYPE_DEPOSIT_REFUND);
            rechargeVoList.add(rechargeVo);
        });
        //将rechargeVoList根据payTime倒序
        rechargeVoList.sort(Comparator.comparing(RechargeVo::getPayTime).reversed());
        return rechargeVoList;
    }

    /**
     * 获取当前正在骑行中的订单
     * @param sn
     * @return
     */
    @Override
    public EtOrder getCurrentOrder(String sn) {
        if(ObjectUtil.isNotNull(asDeviceService.selectAsDeviceBySn(sn))){
            EtOrder etOrder = new EtOrder();
            etOrder.setSn(sn);
            etOrder.setStatus(ServiceConstants.ORDER_STATUS_RIDING);
            List<EtOrder> etOrders = etOrderMapper.selectEtOrderList(etOrder);
            if(ObjectUtil.isNotEmpty(etOrders)){
                return etOrders.get(0);
            }
        }
        return null;
    }

    private boolean toBePaid(String[] statusList) {
        boolean hasOne = false;
        boolean hasThree = false;
        for (String status : statusList) {
            if ("1".equals(status)) {
                hasOne = true;
            } else if ("3".equals(status)) {
                hasThree = true;
            }
            // 如果已经找到了1和3，无需继续遍历
            if (hasOne && hasThree) {
                break;
            }
        }
        // 返回是否同时包含1和3
        return hasOne && hasThree;
    }

    private boolean ended(String[] statusList) {
        // 长度为1表示只有一个元素
        if (statusList.length != 1) {
            return false;
        }
        // 检查唯一元素的值是否为4
        return "4".equals(statusList[0]);
    }


    /**
     * 新增订单
     *
     * @param etOrder 订单
     * @return 结果
     */
    @Override
    public int insertEtOrder(EtOrder etOrder)
    {
        etOrder.setCreateTime(DateUtils.getNowDate());
        return etOrderMapper.insertEtOrder(etOrder);
    }

    /**
     * 修改订单
     *
     * @param etOrder 订单
     * @return 结果
     */
    @Override
    public int updateEtOrder(EtOrder etOrder)
    {
        return etOrderMapper.updateEtOrder(etOrder);
    }

    /**
     * 根据订单号修改订单
     *
     * @param etOrder 订单
     * @return 结果
     */
    @Override
    public int updateEtOrderByOrderNo(EtOrder etOrder)
    {
        return etOrderMapper.updateEtOrderByOrderNo(etOrder);
    }

    /**
     * 批量删除订单
     *
     * @param orderIds 需要删除的订单主键
     * @return 结果
     */
    @Override
    public int deleteEtOrderByOrderIds(Long[] orderIds)
    {
        return etOrderMapper.deleteEtOrderByOrderIds(orderIds);
    }

    /**
     * 删除订单信息
     *
     * @param orderId 订单主键
     * @return 结果
     */
    @Override
    public int deleteEtOrderByOrderId(Long orderId)
    {
        return etOrderMapper.deleteEtOrderByOrderId(orderId);
    }

    /**
     * 预下单
     * 类型 1骑行 2预约 4押金
     * 获取到订单信息后，计算金额，调用微信支付接口，返回预支付信息
     */
    @Transactional
    @SneakyThrows
    @Override
    public PrepayWithRequestPaymentResponse preOrder(@RequestBody EtOrderVo order) {
        String payType = order.getType();
        if(ObjectUtils.isEmpty(payType)){
            throw new ServiceException("订单类型不能为空");
        }
        //这里的订单号只有押金订单才需要
        EtOrder etOrder = null;
        //根据订单号查订单
        if(payType.equals("4")){
            log.info("【预下单】支付场景为：押金支付");
            // TODO 是否存在押金订单，已经存在则不需要充值
            if(isExitDepositOrder(order.getUserId())){
                throw new ServiceException("您已经充值押金，请勿重复充值");
            }
            String orderNo = IdUtils.randomUUID2();
            // 押金订单，直接生成订单
            etOrder = createOrder(order, orderNo);
            int i = etOrderMapper.insertEtOrder(etOrder);
            if(i == 0){
                throw new ServiceException("订单生成失败");
            }
        }else if(payType.equals("2")){
            log.info("【预下单】支付场景为：取消预约支付");
            etOrder = etOrderMapper.selectEtOrderByOrderNo(order.getOrderNo());
            etOrder.setTotalFee(etOrder.getAppointmentFee());
            int updateEtOrder = etOrderService.updateEtOrder(etOrder);
            if(updateEtOrder==0){
                throw new ServiceException("【预下单】订单信息更新失败");
            }
        }else{
            log.info("【预下单】支付场景为：骑行支付");
            etOrder = etOrderMapper.selectEtOrderByOrderNo(order.getOrderNo());
        }
        //TODO 计算订单金额，更新订单金额  还要根据车辆定位，
        // 判断是否在停车区内，
        //      如果在停车区，正常结算费用（根据距离和距离收费，计算费用）
        //      如果不在停车区，加上管理费，
        // 再判断是否在运营区内
        //      如果在运营区内，
        //      如果不在运营区内，加上调度费
        // 根据下单时间后24小时内封顶200元  完成
        //
        //TODO 订单距离根据定位计算，订单时长根据时间计算
//        etOrder.setDistance(order.getDistance());//距离：米
//        etOrder.setDuration(order.getDuration());//订单时长：以秒为单位

        PrepayWithRequestPaymentResponse response = wxPayService.prepayWithRequestPayment(payType,etOrder);
        PrepayWithRequestPaymentResponseVo responseVo = new PrepayWithRequestPaymentResponseVo();
        BeanUtils.copyProperties(response,responseVo);
        responseVo.setSessionId(IdUtils.randomUUIDByDigit(8));
        return responseVo;
    }

    /**
     * 是否已充值过押金
     * @param userId
     * @return
     */
    private boolean isExitDepositOrder(Long userId) {
        AsUser asUser = asUserService.selectUserById(userId);
        return asUser.getBalance().compareTo(BigDecimal.ZERO)!=0;
    }

    @Override
    public EtOrder createOrder(EtOrderVo order,String orderNo) {
        AsUser user = asUserService.selectUserById(order.getUserId());
        EtOrder etOrder = new EtOrder();
        etOrder.setOrderNo(orderNo);
        etOrder.setUserId(user.getUserId());
        etOrder.setUserName(user.getUserName());
        etOrder.setMark(order.getMark());
        etOrder.setCreateTime(DateUtils.getNowDate());
        etOrder.setRuleId(order.getRuleId());
        /** 订单类型 1骑行 2预约 3套餐 4押金 根据类型不同下单的参数也不同，不同的参数有： 金额、类型、rule_id（套餐id）*/
        String type = order.getType();
        if(type.equals("1")){//骑行订单,正常骑行（包含预约费），计时收费，根据开锁时间、起步价和起步时长、时长费和时长分钟等参数，如果有预约，还要加上预约费计算费用
            setAreaInfo(order.getSn(), etOrder);
            etOrder.setType(ServiceConstants.ORDER_TYPE_RIDING);
            etOrder.setStatus(ServiceConstants.ORDER_STATUS_RIDING);
            etOrder.setUnlockTime(DateUtils.getNowDate());
        }else if(order.getType().equals("2")){//预约，创建初始订单，根据开始预约时间结算费用，开始骑行时，结算预约费用，在订单表中通过appointment_start_time字段是否有值或者status是1（取消预约）和paid=1，来判断是否是预约订单
            setAreaInfo(order.getSn(), etOrder);
            etOrder.setType(ServiceConstants.ORDER_TYPE_RIDING);
            etOrder.setStatus(ServiceConstants.ORDER_STATUS_IN_APPOINTMENT);//预约中
            etOrder.setAppointmentStartTime(DateUtils.getNowDate());//预约开始时间
        }else if(order.getType().equals("4")){//押金，创建初始订单，根据计费规则，结算费用，
            etOrder.setType(ServiceConstants.ORDER_TYPE_DEPOSIT);
            EtOperatingArea area = etOperatingAreaService.selectEtOperatingAreaByAreaId(user.getAreaId());
            BigDecimal deposit = new BigDecimal(area.getDeposit());
            etOrder.setTotalFee(deposit);
        }else {
            throw new ServiceException("订单类型错误");
        }
        return etOrder;
    }

    private void verify(EtOrderVo order, Long userId) {
        if(etOrderService.isInOrder(userId,null).size()>0){
           throw new ServiceException("您有未完成的订单，请先完成订单");
        }
        //根据余额和充值记录判断是否有充值过押金，没有充值过押金，提示充值押金
        if(!asUserService.checkIsDeposit(userId)){
            throw new ServiceException("您还未充值押金，请先充值押金");
        }
        //检验预约订单只能有一个
        if(etOrderService.checkIsUnique(userId)){
            throw new ServiceException("您有预约中的订单，请务重复操作");
        }
        AsDevice asDevice = asDeviceService.selectAsDeviceBySn(order.getSn());
        String status = asDevice.getStatus();
        if(!ServiceConstants.VEHICLE_STATUS_NORMAL.equals(status)){
            //根据状态值返回不同的提示
            throw new ServiceException(CommonUtil.format(status));
        }
    }

    /**
     * 检验预约订单只能有一个
     */
    @Override
    public Boolean checkIsUnique(Long userId) {
        int i = etOrderMapper.checkIsUnique(userId);
        return i>0;
    }

    /**
     * 运营数据
     */
    @Override
    public OperatingDataVo getOperatingData(String timeStart,String timeEnd) {
        OperatingDataVo operatingDataVo = new OperatingDataVo();
        if(StrUtil.isNotBlank(timeStart) && StrUtil.isNotBlank(timeEnd)){
            /*收入相关*/
            OperatingDataVo.Income income = new OperatingDataVo.Income();
            income.setTotalIncome(etOrderMapper.getTotalIncome(timeStart,timeEnd));//总收入
            income.setTotalUnpaid(etOrderMapper.getTotalUnpaid(timeStart,timeEnd));//累计待支付
            income.setTotalPaid(etOrderMapper.getTotalPaid(timeStart,timeEnd));//已支付
            income.setTotalRefund(etOrderMapper.getTotalRefund(timeStart,timeEnd));//已退款
            income.setTotalRidingFee(etOrderMapper.getTotalRidingFee(timeStart,timeEnd));//骑行已支付
            income.setTotalRidingRefund(etOrderMapper.getTotalRidingRefund(timeStart,timeEnd));//骑行已退款
            income.setTotalDispatchFee(etOrderMapper.getTotalDispatchFee(timeStart,timeEnd));//调度费已支付
            income.setTotalDispatchRefund(etOrderMapper.getTotalDispatchRefund(timeStart,timeEnd));//调度费已退款
            income.setTotalAppointmentFee(etOrderMapper.getTotalAppointmentFee(timeStart,timeEnd));//预约费已支付
            income.setTotalAppointmentRefund(etOrderMapper.getTotalAppointmentRefund(timeStart,timeEnd));//预约费已退款
            income.setTotalManageFee(etOrderMapper.getTotalManageFee(timeStart,timeEnd));//管理费已支付
            income.setTotalManageRefund(etOrderMapper.getTotalManageRefund(timeStart,timeEnd));//管理费已退款
            operatingDataVo.setIncome(income);

            /*订单相关*/
            OperatingDataVo.OrderVo orderVo = new OperatingDataVo.OrderVo();
            orderVo.setPaidOrder(etOrderMapper.getPaidOrder(timeStart,timeEnd));//已支付订单
            orderVo.setRidingOrder(etOrderMapper.getRidingOrder(timeStart,timeEnd));//骑行中订单
            orderVo.setRefundOrder(etOrderMapper.getRefundOrder(timeStart,timeEnd));//退款订单
            orderVo.setUnpaidOrder(etOrderMapper.getUnpaidOrder(timeStart,timeEnd));//待支付订单
            operatingDataVo.setOrder(orderVo);

            /*设备相关*/
            OperatingDataVo.DeviceVo deviceVo = new OperatingDataVo.DeviceVo();
            deviceVo.setInOperationDevice(deviceService.getInOperationDevice(timeStart,timeEnd));//运营中车辆
            deviceVo.setInOrderDevice(deviceService.getInOrderDevice(timeStart,timeEnd));//有订单车辆
            deviceVo.setNoOrderDevice(deviceService.getNoOrderDevice(timeStart,timeEnd));//无订单车辆
            operatingDataVo.setDevice(deviceVo);

            /*用户相关*/
            OperatingDataVo.UserVo userVo = new OperatingDataVo.UserVo();
            userVo.setTotalUser(asUserService.getTotalUser(timeStart,timeEnd));//总用户
            userVo.setNewUser(asUserService.getNewUser(timeStart,timeEnd));//新增用户
            userVo.setLeaseUser(asUserService.getLeaseUser(timeStart,timeEnd));//租赁用户
            operatingDataVo.setUser(userVo);
            return operatingDataVo;
        }else{
            throw new ServiceException("请选择时间");
        }
    }

    /**
     * 收入对账
     * type:1-按日期 2-按车辆
     */
    @SneakyThrows
    @Override
    public ReconciliationVo reconciliation(String timeStart, String timeEnd, String type,String sn) {
        ReconciliationVo reconciliationVo = new ReconciliationVo();
        List<ReconciliationVo.Reconciliation> reconciliations = new ArrayList<>();
        if("1".equals(type)){//按日期
            // 格式化日期
            SimpleDateFormat dateFormat = new SimpleDateFormat("yyyy-MM-dd");
            int page = 1;
            int limit = DateUtils.differentDaysByMillisecond(timeStart, timeEnd)+1;

            // 获取当前日期
            Calendar calendar = Calendar.getInstance();
            Date date1 = dateFormat.parse(timeEnd);
            calendar.setTime(date1);

            // 计算起始日期
            calendar.add(Calendar.DATE, -(page - 1) * limit);
            Date startDate = calendar.getTime();

            for (int i = 0; i < limit; i++) {
                String formattedDate = dateFormat.format(startDate);
                String startDateStr = formattedDate + " "+ Constants.DATE_FORMAT_START_PEREND;
                String endDateStr = formattedDate + " " +Constants.DATE_FORMAT_END_PEREND;
                if(StrUtil.isNotBlank(timeStart) && StrUtil.isNotBlank(timeEnd)){
                    //获取当前日期的订单
                    ReconciliationVo.Reconciliation reconciliation = new ReconciliationVo.Reconciliation();
                    reconciliation.setDay(formattedDate);
                    BigDecimal payFee = etOrderMapper.getPayFee(startDateStr, endDateStr, null);
                    BigDecimal defaultPayFee = payFee != null ? payFee : BigDecimal.ZERO;
                    reconciliation.setPayFee(defaultPayFee);
                    BigDecimal refundFee = etOrderMapper.getRefundFee(startDateStr, endDateStr, null);
                    BigDecimal defaultRefundFee = refundFee != null ? refundFee : BigDecimal.ZERO;
                    reconciliation.setRefundFee(defaultRefundFee);
                    BigDecimal incomeFee = etOrderMapper.getIncome(startDateStr, endDateStr, null);
                    BigDecimal defaultIncomeFeeFee = incomeFee != null ? incomeFee : BigDecimal.ZERO;
                    reconciliation.setIncome(defaultIncomeFeeFee);
                    reconciliations.add(reconciliation);
                }
                // 日期减一天
                calendar.add(Calendar.DATE,-1);
                startDate = calendar.getTime();
            }
            reconciliationVo.setReconciliationList(reconciliations);
            //计算总收入   payFee
            // 使用Java 8 Stream API来计算payFee字段的总和
            BigDecimal totalPayFee = reconciliations != null && !reconciliations.isEmpty()
                    ? reconciliations.stream()
                    .map(ReconciliationVo.Reconciliation::getPayFee)
                    .reduce(BigDecimal.ZERO, BigDecimal::add)
                    : null;
            reconciliationVo.setTotalPayFee(totalPayFee);

            BigDecimal totalRefundFee = reconciliations != null && !reconciliations.isEmpty()
                    ? reconciliations.stream()
                    .map(ReconciliationVo.Reconciliation::getRefundFee)
                    .reduce(BigDecimal.ZERO, BigDecimal::add)
                    : null;
            reconciliationVo.setTotalRefundFee(totalRefundFee);

            BigDecimal totalServiceFee = reconciliations != null && !reconciliations.isEmpty()
                    ? reconciliations.stream()
                    .map(ReconciliationVo.Reconciliation::getServiceFee)
                    .reduce(BigDecimal.ZERO, BigDecimal::add)
                    : null;
            reconciliationVo.setTotalServiceFee(totalServiceFee);

            BigDecimal totalIncome = reconciliations != null && !reconciliations.isEmpty()
                    ? reconciliations.stream()
                    .map(ReconciliationVo.Reconciliation::getIncome)
                    .reduce(BigDecimal.ZERO, BigDecimal::add)
                    : null;
            reconciliationVo.setTotalIncome(totalIncome);

            return reconciliationVo;
        }else {
            if(StrUtil.isBlank(sn)){
               throw new ServiceException("类型为2时，车辆的sn必传");
            }
            AsDevice asDevice = new AsDevice();
            asDevice.setSn(sn);
            // 按车辆,先把所有车辆查出，如果参数中有车辆，就查出该车辆的按日期的订单
            List<AsDevice> asDevices = asDeviceService.selectAsDeviceList(asDevice);
            for(AsDevice asDevice1:asDevices){
                String sn1 = asDevice1.getSn();
                if(StrUtil.isNotBlank(sn1)){
                    //获取当前日期的订单
                    ReconciliationVo.Reconciliation reconciliation = new ReconciliationVo.Reconciliation();
                    reconciliation.setSn(sn1);
                    reconciliation.setPayFee(etOrderMapper.getPayFee(timeStart,timeEnd,sn1));
                    reconciliation.setRefundFee(etOrderMapper.getRefundFee(timeStart,timeEnd,sn1));
                    reconciliation.setIncome(etOrderMapper.getIncome(timeStart,timeEnd,sn1));
                    reconciliations.add(reconciliation);
                }
            }
            reconciliationVo.setReconciliationList(reconciliations);
            return reconciliationVo;
        }
    }

    /**
     * 最近一笔订单
     */
    @Override
    public EtOrder latestOrder(EtOrder etOrder) {
        EtOrder order = etOrderMapper.selectLatestOrder(etOrder);
        return order;
    }

    /**
     * 退款
     */
    @Transactional
    @Override
    public int refund(EtOrder etOrder) {
        //根据订单号查询订单信息
        EtOrder etOrder1 = etOrderService.selectEtOrderByOrderNo(etOrder.getOrderNo());
        if(ObjectUtil.isNull(etOrder1)){
            throw new ServiceException("订单不存在");
        }
        if(!ServiceConstants.ORDER_TYPE_RIDING.equals(etOrder1.getType())){
            throw new ServiceException("退款失败，类型必须是骑行订单");
        }
        if(!ServiceConstants.ORDER_STATUS_ORDER_END.equals(etOrder1.getStatus())){
            throw new ServiceException("退款失败，订单未结束");
        }
        if(ServiceConstants.ORDER_PAY_STATUS_NON_PAYMENT.equals(etOrder1.getPaid())){
            throw new ServiceException("订单未支付，不能退款");
        }
        /** 1.退款*/
        //退款金额
        BigDecimal refundAmount = new BigDecimal("0");
        BigDecimal appointmentFee = etOrder.getAppointmentFee();
        if(ObjectUtil.isNotNull(appointmentFee) && !appointmentFee.equals(BigDecimal.ZERO)){
            refundAmount = refundAmount.add(appointmentFee);
        }
        BigDecimal dispatchFee = etOrder.getDispatchFee();
        if(ObjectUtil.isNotNull(dispatchFee) && !dispatchFee.equals(BigDecimal.ZERO)){
            refundAmount = refundAmount.add(dispatchFee);
        }
        BigDecimal manageFee = etOrder.getManageFee();
        if(ObjectUtil.isNotNull(manageFee) && !manageFee.equals(BigDecimal.ZERO)){
            refundAmount = refundAmount.add(manageFee);
        }
        BigDecimal ridingFee = etOrder.getRidingFee();
        if(ObjectUtil.isNotNull(ridingFee) && !ridingFee.equals(BigDecimal.ZERO)){
            refundAmount = refundAmount.add(ridingFee);
        }
        log.info("总金额：【{}】，退款金额：【{}】", etOrder1.getTotalFee(), refundAmount);
        Refund refund = wxPayService.refund(etOrder1, etOrder.getReason(), refundAmount);
        /** 2.记录退款表  创建退款对象*/
        EtRefund refund1= createRefund(etOrder, refundAmount, appointmentFee, dispatchFee, manageFee, ridingFee, refund,ServiceConstants.REFUND_TYPE_SYSTEM);
        int i = etRefundService.insertEtRefund(refund1);
        if(i>0){
            log.info("保存退款对象成功");
        }
        return i;
    }

    /**
     * 提现
     */
    @Transactional
    @Override
    public int withdraw(EtOrder etOrder1, BigDecimal deposit) {
        Refund refund = wxPayService.refund(etOrder1, "押金提现",deposit);
        /** 2.记录退款表  创建退款对象*/
        etOrder1.setReason("押金提现");
        EtRefund refund1= createRefund(etOrder1, deposit, null, null, null, null, refund,ServiceConstants.REFUND_TYPE_DEPOSIT);
        int i = etRefundService.insertEtRefund(refund1);
        if(i>0){
            log.info("【提现金额】保存退款对象成功");
        }
        return i;
    }

    @Override
    public EtRefund createRefund(EtOrder etOrder, BigDecimal refundAmount, BigDecimal appointmentFee, BigDecimal dispatchFee, BigDecimal manageFee, BigDecimal ridingFee, Refund refund,String type) {
        EtRefund etRefund = new EtRefund();
        etRefund.setAmount(refundAmount);
        etRefund.setReason(etOrder.getReason());
        etRefund.setAppointmentFee(appointmentFee);
        etRefund.setDispatchFee(dispatchFee);
        etRefund.setManageFee(manageFee);
        etRefund.setRidingFee(ridingFee);
        etRefund.setOrderNo(etOrder.getOrderNo());
        etRefund.setUserId(etOrder.getUserId());
        etRefund.setRefundNo(refund.getOutRefundNo());
        etRefund.setType(type);
        StringBuilder itemDesc = new StringBuilder();
        if(ObjectUtil.isNotNull(appointmentFee) && !appointmentFee.equals(BigDecimal.ZERO)){
            itemDesc.append("预约费:").append(appointmentFee).append("元,");
        }
        if(ObjectUtil.isNotNull(dispatchFee) && !dispatchFee.equals(BigDecimal.ZERO)){
            itemDesc.append("调度费:").append(dispatchFee).append("元,");
        }
        if(ObjectUtil.isNotNull(manageFee) && !manageFee.equals(BigDecimal.ZERO)){
            itemDesc.append("管理费:").append(manageFee).append("元,");
        }
        if(ObjectUtil.isNotNull(ridingFee) && !ridingFee.equals(BigDecimal.ZERO)){
            itemDesc.append("骑行费:").append(ridingFee).append("元,");
        }
        etRefund.setItemDesc(itemDesc.toString());
        return etRefund;
    }

    private void setAreaInfo(String sn, EtOrder etOrder) {
        AsDevice asDevice = deviceService.selectAsDeviceBySn(sn);
        EtOperatingArea etOperatingArea = etOperatingAreaService.selectEtOperatingAreaByAreaId(asDevice.getAreaId());
        etOrder.setAreaId(etOperatingArea.getAreaId());
        etOrder.setArea(etOperatingArea.getAreaName());
        etOrder.setSn(sn);
    }

    /**
     * 查询当前用户是否有正在进行中的订单
     */
    @Override
    public List<EtOrder> isInOrder(Long userId,String orderNo) {
        List<EtOrder> inOrder = etOrderMapper.isInOrder(userId, orderNo);
        for(EtOrder order:inOrder){
            EtFeeRule etFeeRule = etFeeRuleService.selectEtFeeRuleByRuleId(order.getRuleId());
            if(ObjectUtil.isNotNull(etFeeRule)){
                order.setRule(etFeeRule);
            }
        }
        return inOrder;
    }
}
