package com.ruoyi.system.service.impl;

import java.util.List;
import com.ruoyi.common.utils.DateUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;
import com.ruoyi.system.mapper.EtRefundMapper;
import com.ruoyi.system.domain.EtRefund;
import com.ruoyi.system.service.IEtRefundService;

/**
 * 退款订单Service业务层处理
 *
 * @author 邱贞招
 * @date 2024-05-10
 */
@Service
public class EtRefundServiceImpl implements IEtRefundService
{
    @Autowired
    private EtRefundMapper etRefundMapper;

    /**
     * 查询退款订单
     *
     * @param id 退款订单主键
     * @return 退款订单
     */
    @Override
    public EtRefund selectEtRefundById(Long id)
    {
        return etRefundMapper.selectEtRefundById(id);
    }

    /**
     * 根据订单号查询退款订单对象
     *
     * @param orderNo 订单号
     * @return 退款订单
     */
    @Override
    public EtRefund selectEtRefundByOrderNo(String orderNo)
    {
        return etRefundMapper.selectEtRefundByOrderNo(orderNo);
    }

    /**
     * 根据退款单号查询退款订单对象
     *
     * @param refundNo 退款订号
     * @return 退款订单
     */
    @Override
    public EtRefund selectEtRefundByRefundNo(String refundNo)
    {
        return etRefundMapper.selectEtRefundByRefundNo(refundNo);
    }


    /**
     * 查询退款订单列表
     *
     * @param etRefund 退款订单
     * @return 退款订单
     */
    @Override
    public List<EtRefund> selectEtRefundList(EtRefund etRefund)
    {
        return etRefundMapper.selectEtRefundList(etRefund);
    }

    /**
     * 新增退款订单
     *
     * @param etRefund 退款订单
     * @return 结果
     */
    @Override
    public int insertEtRefund(EtRefund etRefund)
    {
        etRefund.setCreateTime(DateUtils.getNowDate());
        return etRefundMapper.insertEtRefund(etRefund);
    }

    /**
     * 修改退款订单
     *
     * @param etRefund 退款订单
     * @return 结果
     */
    @Override
    public int updateEtRefund(EtRefund etRefund)
    {
        return etRefundMapper.updateEtRefund(etRefund);
    }

    /**
     * 批量删除退款订单
     *
     * @param ids 需要删除的退款订单主键
     * @return 结果
     */
    @Override
    public int deleteEtRefundByIds(Long[] ids)
    {
        return etRefundMapper.deleteEtRefundByIds(ids);
    }

    /**
     * 删除退款订单信息
     *
     * @param id 退款订单主键
     * @return 结果
     */
    @Override
    public int deleteEtRefundById(Long id)
    {
        return etRefundMapper.deleteEtRefundById(id);
    }

    /**
     * 根据退款单号修改退款订单
     * @param etRefund
     * @return
     */
    @Override
    public int updateEtRefundByRefundNo(EtRefund etRefund) {
        return etRefundMapper.updateEtRefundByRefundNo(etRefund);
    }
}
