package com.ruoyi.system.service.impl;

import com.alibaba.fastjson2.JSON;
import com.ruoyi.common.config.WxPayConfig;
import com.ruoyi.common.constant.ServiceConstants;
import com.ruoyi.common.core.domain.entity.AsUser;
import com.ruoyi.common.core.domain.model.LoginUser;
import com.ruoyi.common.core.redis.RedisCache;
import com.ruoyi.common.core.redis.RedisLock;
import com.ruoyi.common.exception.ServiceException;
import com.ruoyi.common.utils.SecurityUtils;
import com.ruoyi.common.utils.ServiceUtil;
import com.ruoyi.common.utils.uuid.IdUtils;
import com.ruoyi.system.domain.EtOrder;
import com.ruoyi.system.domain.vo.AttachVo;
import com.ruoyi.system.service.IAsUserService;
import com.ruoyi.system.service.IEtOperatingAreaService;
import com.ruoyi.system.service.IEtOrderService;
import com.ruoyi.system.service.IWxPayService;
import com.wechat.pay.java.core.notification.NotificationParser;
import com.wechat.pay.java.core.notification.RequestParam;
import com.wechat.pay.java.service.payments.jsapi.JsapiService;
import com.wechat.pay.java.service.payments.jsapi.JsapiServiceExtension;
import com.wechat.pay.java.service.payments.jsapi.model.*;
import com.wechat.pay.java.service.payments.model.Transaction;
import com.wechat.pay.java.service.profitsharing.ProfitsharingService;
import com.wechat.pay.java.service.profitsharing.model.*;
import com.wechat.pay.java.service.refund.RefundService;
import com.wechat.pay.java.service.refund.model.AmountReq;
import com.wechat.pay.java.service.refund.model.CreateRequest;
import com.wechat.pay.java.service.refund.model.Refund;
import lombok.extern.slf4j.Slf4j;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;
import org.springframework.transaction.support.TransactionTemplate;

import javax.management.relation.RelationType;
import javax.servlet.http.HttpServletRequest;
import java.math.BigDecimal;
import java.util.ArrayList;
import java.util.List;

/**
 * 微信支付服务
 * @author 辉
 * 2024/3/11
 */
@Service
@Slf4j
public class WxPayService implements IWxPayService {

    @Autowired
    private JsapiService jsapiService;

    @Autowired
    private JsapiServiceExtension jsapiServiceExtension;

    @Autowired
    private WxPayConfig wxPayConfig;

    @Autowired
    private IEtOperatingAreaService etOperatingAreaService;

    @Autowired
    private IEtOrderService etOrderService;

    @Autowired
    private IAsUserService asUserService;

    @Autowired
    private NotificationParser notificationParser;

    @Autowired
    private TransactionTemplate transactionTemplate;

    @Autowired
    public RefundService refundService2;

    @Autowired
    public ProfitsharingService profitsharingService;

    @Autowired
    private RedisLock redisLock;

    private static final String CNY = "CNY";

    private static final String PREPAY_LOCK = "prepay:";

    @Override
    public PrepayWithRequestPaymentResponse prepayWithRequestPayment(String payType,EtOrder order) {
        String billNo = order.getOrderNo();
        // 创建订单
        AsUser user = asUserService.selectUserById(order.getUserId());
        try {
            // 获取JSAPI所需参数
            PrepayRequest request = new PrepayRequest();
            request.setAmount(getAmount(order.getTotalFee()));
            String outTradeNo = IdUtils.getOrderNo("wx");
            order.setOutTradeNo(outTradeNo);
            int updateEtOrder = etOrderService.updateEtOrder(order);
            if(updateEtOrder == 0){
                throw new ServiceException("更新订单outTradeNo失败");
            }
            request.setOutTradeNo(outTradeNo);
            request.setAppid(wxPayConfig.getAppId());
            request.setMchid(wxPayConfig.getMerchantId());
            String type = order.getType();
            String description = type.equals(ServiceConstants.ORDER_TYPE_RIDING) ? "骑行订单-"+billNo : "押金充值-"+billNo;
            LoginUser loginUser = SecurityUtils.getLoginUser();
            log.info("【预下单】获取登录用户信息:"+JSON.toJSONString(loginUser));
            request.setAttach(JSON.toJSONString(new AttachVo(payType,user.getUserId(), loginUser.getToken())));
            request.setDescription(description);
            request.setNotifyUrl(wxPayConfig.getNotifyUrl());
            request.setPayer(getPayer(user.getWxopenid()));
            SettleInfo settleInfo = new SettleInfo();
            settleInfo.setProfitSharing(Boolean.TRUE);
            request.setSettleInfo(settleInfo);
            PrepayWithRequestPaymentResponse res = jsapiServiceExtension.prepayWithRequestPayment(request);
            return res;
        } finally {
            redisLock.unlock(PREPAY_LOCK + billNo);
        }
    }

    /**
     * 关闭支付订单
     * @param billNo 平台订单编号
     */
    @Override
    public void closeOrder(String billNo) {
        CloseOrderRequest request = new CloseOrderRequest();
        request.setMchid(wxPayConfig.getMerchantId());
        request.setOutTradeNo(billNo);
        jsapiService.closeOrder(request);
    }

    @Override
    public Transaction queryOrderById(String prePayId) {
        QueryOrderByIdRequest request = new QueryOrderByIdRequest();
        request.setMchid(wxPayConfig.getMerchantId());
        request.setTransactionId(prePayId);
        return jsapiService.queryOrderById(request);
    }

    @Override
    public Transaction queryOrderByOutTradeNo(String billNo) {
        QueryOrderByOutTradeNoRequest request = new QueryOrderByOutTradeNoRequest();
        request.setMchid(wxPayConfig.getMerchantId());
        request.setOutTradeNo(billNo);
        return jsapiService.queryOrderByOutTradeNo(request);
    }

    /**
     * 退款
     * @param etOrder 原订单对象
     * @param reason 退款原因
     * @param amount 退款金额
     */
    @Override
    public Refund refund(EtOrder etOrder,String reason,BigDecimal amount) {
        CreateRequest request = new CreateRequest();
        request.setOutTradeNo(etOrder.getOutTradeNo());
        request.setOutRefundNo(IdUtils.randomUUID2());
        request.setReason(reason);
        request.setAmount(getAmountReq(etOrder.getTotalFee(),amount));
        request.setNotifyUrl(wxPayConfig.getRefundNotifyUrl());
        log.info("【退款】请求微信参数:【{}】",JSON.toJSONString(request));
        return refundService2.create(request);
    }

    /** 请求分账API */
    public OrdersEntity createOrder(String transactionId,List<CreateOrderReceiver> receivers) {
        CreateOrderRequest request = new CreateOrderRequest();
        request.setAppid(wxPayConfig.getAppId());
        request.setTransactionId(transactionId);// 微信订单号
        request.setOutOrderNo(IdUtils.getOrderNo("fz"));// 商户系统内部分账单号
        request.setReceivers(receivers);
        request.setUnfreezeUnsplit(false);
        return profitsharingService.createOrder(request);
    }

    /** 添加分账接收方 */
    @Override
    public AddReceiverResponse addReceiver(String wxopenid) {
        AddReceiverRequest request = new AddReceiverRequest();
        request.setAppid(wxPayConfig.getAppId());
        request.setType(ReceiverType.PERSONAL_OPENID);
        request.setAccount(wxopenid);
        request.setRelationType(ReceiverRelationType.PARTNER);
        return profitsharingService.addReceiver(request);
    }

    /** 删除分账接收方 */
    public DeleteReceiverResponse deleteReceiver(String wxopenid) {
        DeleteReceiverRequest request = new DeleteReceiverRequest();
        request.setAppid(wxPayConfig.getAppId());
        request.setType(ReceiverType.PERSONAL_OPENID);
        request.setAccount(wxopenid);
        return profitsharingService.deleteReceiver(request);
    }


    /**
     * 验签并解析
     * @param request 请求
     * @param body 请求体
     * @param clazz 返回值类型
     */
    private <T> T checkAndParse(HttpServletRequest request, String body, Class<T> clazz) {
        // 构造 RequestParam
        RequestParam requestParam = new RequestParam.Builder()
                .serialNumber(request.getHeader("Wechatpay-Serial"))
                .nonce(request.getHeader("Wechatpay-Nonce"))
                .signature(request.getHeader("Wechatpay-Signature"))
                .timestamp(request.getHeader("Wechatpay-Timestamp"))
                .body(body)
                .build();
        // 验签
        return notificationParser.parse(requestParam, clazz);
    }

    private Payer getPayer(String openId) {
        Payer payer = new Payer();
        payer.setOpenid(openId);
        return payer;
    }

    private Amount getAmount(BigDecimal money) {
        Amount amount = new Amount();
        amount.setTotal(money.multiply(new BigDecimal(100)).intValue());
        amount.setCurrency(CNY);
        return amount;
    }

    private AmountReq getAmountReq(BigDecimal total,BigDecimal rerundAmount) {
        AmountReq amount = new AmountReq();
        amount.setTotal(total.multiply(new BigDecimal(100)).longValue());
        amount.setRefund(rerundAmount.multiply(new BigDecimal(100)).longValue());
        amount.setCurrency(CNY);
        return amount;
    }


}
